export declare class RationalTime {
    private readonly epsilon;
    value: number;
    rate: number;
    constructor(value?: number, rate?: number);
    /**
     * Determines if the current `RationalTime` instance is almost equal to another `RationalTime` instance
     * within a specified delta.
     *
     * @param other - The other `RationalTime` instance to compare against.
     * @param delta - The maximum allowable difference in seconds for the two times to be considered almost equal.
     *                 Defaults to the value of `this.epsilon`.
     * @returns `true` if the absolute difference between the two times is less than or equal to `delta`, otherwise `false`.
     */
    almostEqual(other: RationalTime, delta?: number): boolean;
    /**
     * Rounds the value of the current `RationalTime` instance up to the nearest integer.
     *
     * @returns {RationalTime} A new `RationalTime` instance with the value rounded up.
     */
    ceil(): RationalTime;
    /**
     * Calculates the duration between two RationalTime instances.
     *
     * @param startTime - The starting time.
     * @param endTimeExclusive - The ending time (exclusive).
     * @returns A new RationalTime instance representing the duration between the start and end times.
     */
    static durationFromStartEndTime(startTime: RationalTime, endTimeExclusive: RationalTime): RationalTime;
    /**
     * Calculates the duration between two RationalTime instances, inclusive of the end time.
     *
     * @param startTime - The starting RationalTime instance.
     * @param endTimeInclusive - The ending RationalTime instance, inclusive.
     * @returns A new RationalTime instance representing the duration from start to end, inclusive.
     */
    static durationFromStartEndTimeInclusive(startTime: RationalTime, endTimeInclusive: RationalTime): RationalTime;
    /**
     * Rounds the value of the current `RationalTime` instance down to the nearest integer.
     *
     * @returns A new `RationalTime` instance with the floored value and the same rate.
     */
    floor(): RationalTime;
    /**
     * Creates a `RationalTime` instance from the given frame number and rate.
     *
     * @param frame - The frame number to be converted to `RationalTime`.
     * @param rate - The rate at which the frames are being played.
     * @returns A new `RationalTime` instance representing the given frame and rate.
     */
    static fromFrames(frame: number, rate: number): RationalTime;
    static fromSeconds(seconds: number, rate?: number): RationalTime;
    static fromTimeString(timeString: string, rate: number): RationalTime;
    static fromTimecode(timecode: string, rate: number): RationalTime;
    isInvalidTime(): boolean;
    static isValidTimecodeRate(rate: number): boolean;
    static nearestValidTimecodeRate(rate: number): number;
    rescaledTo(newRate: number | RationalTime): RationalTime;
    round(): RationalTime;
    strictlyEqual(other: RationalTime): boolean;
    toFrames(rate?: number): number;
    toNearestTimecode(rate?: number, dropFrame?: boolean | undefined): string;
    toSeconds(): number;
    toTimeString(): string;
    /**
     * Converts the current RationalTime instance to a timecode string.
     *
     * @param rate - The frame rate to use for the conversion. Defaults to the instance's rate.
     * @param dropFrame - Optional boolean indicating whether to use drop-frame timecode. If undefined, the default behavior is used.
     * @returns The timecode string representation of the current RationalTime instance.
     */
    toTimecode(rate?: number, dropFrame?: boolean | undefined): string;
    valueRescaledTo(newRate: number | RationalTime): number;
    /**
     * Parses a time string in the format "HH:MM:SS" and converts it to the total number of seconds.
     *
     * @param timeString - The time string to parse, expected in the format "HH:MM:SS".
     * @returns The total number of seconds represented by the time string.
     * @throws Will throw an error if the time string format is invalid.
     */
    private static parseTimeStringToSeconds;
    private static parseTimecodeToFrames;
    private static framesToTimecode;
}
//# sourceMappingURL=RationalTime.d.ts.map