// RationalTime.test.ts
import { describe, it, expect } from "vitest";
import { RationalTime } from "./RationalTime";
describe("RationalTime", () => {
    describe("constructor", () => {
        it("should create an instance with default values", () => {
            const rt = new RationalTime();
            expect(rt.value).toBe(0);
            expect(rt.rate).toBe(1);
        });
        it("should create an instance with provided values", () => {
            const rt = new RationalTime(10, 24);
            expect(rt.value).toBe(10);
            expect(rt.rate).toBe(24);
        });
    });
    describe("almostEqual", () => {
        it("should return true for equal times", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(10, 24);
            expect(rt1.almostEqual(rt2)).toBe(true);
        });
        it("should return true for times within delta", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(10.0000005, 24);
            expect(rt1.almostEqual(rt2)).toBe(true);
        });
        it("should return false for times outside delta", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(10.1, 24);
            expect(rt1.almostEqual(rt2)).toBe(false);
        });
    });
    describe("ceil", () => {
        it("should return a RationalTime with value rounded up", () => {
            const rt = new RationalTime(10.3, 24);
            const rtCeil = rt.ceil();
            expect(rtCeil.value).toBe(11);
            expect(rtCeil.rate).toBe(24);
        });
    });
    describe("floor", () => {
        it("should return a RationalTime with value rounded down", () => {
            const rt = new RationalTime(10.7, 24);
            const rtFloor = rt.floor();
            expect(rtFloor.value).toBe(10);
            expect(rtFloor.rate).toBe(24);
        });
    });
    describe("round", () => {
        it("should return a RationalTime with value rounded to nearest integer", () => {
            const rt = new RationalTime(10.5, 24);
            const rtRound = rt.round();
            expect(rtRound.value).toBe(11);
            expect(rtRound.rate).toBe(24);
        });
    });
    describe("rescaledTo", () => {
        it("should rescale to a new rate", () => {
            const rt = new RationalTime(10, 24);
            const rtRescaled = rt.rescaledTo(48);
            expect(rtRescaled.value).toBe(20);
            expect(rtRescaled.rate).toBe(48);
        });
        it("should rescale using another RationalTime", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(0, 48);
            const rtRescaled = rt1.rescaledTo(rt2);
            expect(rtRescaled.value).toBe(20);
            expect(rtRescaled.rate).toBe(48);
        });
    });
    describe("toFrames", () => {
        it("should convert to frames using current rate", () => {
            const rt = new RationalTime(10, 24);
            expect(rt.toFrames()).toBe(10);
        });
        it("should convert to frames using provided rate", () => {
            const rt = new RationalTime(10, 24);
            expect(rt.toFrames(48)).toBe(20);
        });
    });
    describe("toSeconds", () => {
        it("should convert to seconds", () => {
            const rt = new RationalTime(48, 24);
            expect(rt.toSeconds()).toBe(2);
        });
    });
    describe("toTimeString", () => {
        it("should convert to time string", () => {
            const rt = new RationalTime(3661.5, 1);
            expect(rt.toTimeString()).toBe("01:01:01.50");
        });
    });
    describe("toTimecode", () => {
        it("should convert to timecode string with default rate", () => {
            const rt = new RationalTime(1500, 25);
            expect(rt.toTimecode()).toBe("00:01:00:00");
        });
        it("should convert to timecode with specified rate", () => {
            const rt = new RationalTime(1800, 30);
            expect(rt.toTimecode(30)).toBe("00:01:00:00");
        });
        it("should convert to timecode with drop frame", () => {
            const rt = new RationalTime(1800, 29.97);
            expect(rt.toTimecode(29.97, true)).toBe("00:01:00;02");
        });
    });
    describe("isInvalidTime", () => {
        it("should return true for invalid time (NaN value)", () => {
            const rt = new RationalTime(NaN, 24);
            expect(rt.isInvalidTime()).toBe(true);
        });
        it("should return true for invalid time (zero rate)", () => {
            const rt = new RationalTime(10, 0);
            expect(rt.isInvalidTime()).toBe(true);
        });
        it("should return false for valid time", () => {
            const rt = new RationalTime(10, 24);
            expect(rt.isInvalidTime()).toBe(false);
        });
    });
    describe("isValidTimecodeRate", () => {
        it("should return true for valid rates", () => {
            expect(RationalTime.isValidTimecodeRate(24)).toBe(true);
            expect(RationalTime.isValidTimecodeRate(29.97)).toBe(true);
        });
        it("should return false for invalid rates", () => {
            expect(RationalTime.isValidTimecodeRate(27)).toBe(false);
            expect(RationalTime.isValidTimecodeRate(31)).toBe(false);
        });
    });
    describe("nearestValidTimecodeRate", () => {
        it("should return nearest valid rate for non-standard rate", () => {
            expect(RationalTime.nearestValidTimecodeRate(27)).toBe(25);
            expect(RationalTime.nearestValidTimecodeRate(28)).toBe(29.97);
        });
        it("should return the same rate if it is valid", () => {
            expect(RationalTime.nearestValidTimecodeRate(24)).toBe(24);
        });
    });
    describe("fromFrames", () => {
        it("should create RationalTime from frames and rate", () => {
            const rt = RationalTime.fromFrames(240, 24);
            expect(rt.value).toBe(240);
            expect(rt.rate).toBe(24);
        });
    });
    describe("fromSeconds", () => {
        it("should create RationalTime from seconds and rate", () => {
            const rt = RationalTime.fromSeconds(10, 24);
            expect(rt.value).toBe(240);
            expect(rt.rate).toBe(24);
        });
        it("should default rate to 1 if not provided", () => {
            const rt = RationalTime.fromSeconds(10);
            expect(rt.value).toBe(10);
            expect(rt.rate).toBe(1);
        });
    });
    describe("fromTimeString", () => {
        it("should create RationalTime from time string", () => {
            const rt = RationalTime.fromTimeString("01:00:00", 24);
            expect(rt.value).toBe(86400);
            expect(rt.rate).toBe(24);
        });
    });
    describe("fromTimecode", () => {
        it('should create RationalTime from timecode with ":" delimiter', () => {
            const rt = RationalTime.fromTimecode("01:00:00:00", 24);
            expect(rt.value).toBe(86400);
            expect(rt.rate).toBe(24);
        });
        it('should create RationalTime from timecode with ";" delimiter', () => {
            const rt = RationalTime.fromTimecode("01:00:00;00", 29.97);
            expect(rt.value).toBeCloseTo(107892, 0);
            expect(rt.rate).toBe(29.97);
        });
    });
    describe("strictlyEqual", () => {
        it("should return true for identical RationalTimes", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(10, 24);
            expect(rt1.strictlyEqual(rt2)).toBe(true);
        });
        it("should return false for different values", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(11, 24);
            expect(rt1.strictlyEqual(rt2)).toBe(false);
        });
        it("should return false for different rates", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(10, 25);
            expect(rt1.strictlyEqual(rt2)).toBe(false);
        });
    });
    describe("valueRescaledTo", () => {
        it("should return value rescaled to new rate", () => {
            const rt = new RationalTime(10, 24);
            const valueRescaled = rt.valueRescaledTo(48);
            expect(valueRescaled).toBe(20);
        });
        it("should return value rescaled using another RationalTime", () => {
            const rt1 = new RationalTime(10, 24);
            const rt2 = new RationalTime(0, 48);
            const valueRescaled = rt1.valueRescaledTo(rt2);
            expect(valueRescaled).toBe(20);
        });
    });
    describe("durationFromStartEndTime", () => {
        it("should compute duration excluding end time", () => {
            const startTime = new RationalTime(10, 24);
            const endTimeExclusive = new RationalTime(15, 24);
            const duration = RationalTime.durationFromStartEndTime(startTime, endTimeExclusive);
            expect(duration.value).toBe(5);
            expect(duration.rate).toBe(24);
        });
    });
    describe("durationFromStartEndTimeInclusive", () => {
        it("should compute duration including end time", () => {
            const startTime = new RationalTime(10, 24);
            const endTimeInclusive = new RationalTime(15, 24);
            const duration = RationalTime.durationFromStartEndTimeInclusive(startTime, endTimeInclusive);
            expect(duration.value).toBe(6);
            expect(duration.rate).toBe(24);
        });
    });
    describe("toNearestTimecode", () => {
        it("should convert to nearest timecode with default rate", () => {
            const rt = new RationalTime(1500, 25);
            expect(rt.toNearestTimecode()).toBe("00:01:00:00");
        });
        it("should convert to nearest timecode with specified rate", () => {
            const rt = new RationalTime(1801, 29.97);
            expect(rt.toNearestTimecode(29.97)).toBe("00:01:00:03");
        });
        it("should handle drop frame timecode", () => {
            const rt = new RationalTime(1801, 29.97);
            expect(rt.toNearestTimecode(29.97, true)).toBe("00:01:00;03");
        });
    });
});
//# sourceMappingURL=RationalTime.spec.js.map