// Define the decorator function
import "reflect-metadata";
// #region DTO Decorator
export function dto(target, propertyKey) {
    // Attach a metadata flag
    Reflect.metadata("dto", true)(target, propertyKey);
}
export function isDto(target, propertyKey) {
    return Reflect.getMetadata("dto", target, propertyKey) === true;
}
// #endregion
// #region SubDocument Decorator
// /**
//  * This metadata flag indicates that the sub-document in Firestore replaces the parent.
//  *
//  * @param target - The target object on which the property is defined.
//  * @param propertyKey - The name of the property to which the metadata flag is attached.
//  */
// export function subDocumentElevate(target: any, propertyKey: string) {
//   // Attach a metadata flag
//   Reflect.metadata("subDocumentElevate", true)(target, propertyKey);
// }
// export function isSubDocumentElevate(target: any, propertyKey: string) {
//   return (
//     Reflect.getMetadata("subDocumentElevate", target, propertyKey) === true
//   );
// }
/**
 * This metadata flag indicates that the property is a sub-document in Firestore.
 *
 * @param target - The target object on which the property is defined.
 * @param propertyKey - The name of the property to which the metadata flag is attached.
 */
export function subDocument(target, propertyKey) {
    // Attach a metadata flag
    Reflect.metadata("subDocument", true)(target, propertyKey);
}
export function isSubDocument(target, propertyKey) {
    return Reflect.getMetadata("subDocument", target, propertyKey) === true;
}
export function getSubDocumentProperties(instance, parentKey = "") {
    const subDocumentProperties = [];
    for (const propertyKey of Object.keys(instance)) {
        const fullKey = parentKey ? `${parentKey}.${propertyKey}` : propertyKey;
        if (isSubDocument(instance, propertyKey)) {
            if (Array.isArray(instance[propertyKey])) {
                instance[propertyKey].forEach((_, index) => {
                    subDocumentProperties.push(`${fullKey}[${index}]`);
                });
            }
            else {
                subDocumentProperties.push(fullKey);
            }
        }
        const propertyValue = instance[propertyKey];
        // Check if the property is an object and not null
        if (typeof propertyValue === "object" && propertyValue !== null) {
            if (Array.isArray(propertyValue)) {
                // Handle arrays
                propertyValue.forEach((item, index) => {
                    if (typeof item === "object" && item !== null) {
                        subDocumentProperties.push(...getSubDocumentProperties(item, `${fullKey}[${index}]`));
                    }
                });
            }
            else {
                // Handle nested objects
                subDocumentProperties.push(...getSubDocumentProperties(propertyValue, fullKey));
            }
        }
    }
    return subDocumentProperties;
}
export function getSubDocuments(instance, parentKey = "") {
    const subDocuments = {};
    for (const propertyKey of Object.keys(instance)) {
        const fullKey = parentKey ? `${parentKey}.${propertyKey}` : propertyKey;
        if (isSubDocument(instance, propertyKey)) {
            if (Array.isArray(instance[propertyKey])) {
                instance[propertyKey].forEach((item, index) => {
                    subDocuments[`${fullKey}[${index}]`] = item;
                });
            }
            else {
                subDocuments[fullKey] = instance[propertyKey];
            }
        }
        const propertyValue = instance[propertyKey];
        // Check if the property is an object and not null
        if (typeof propertyValue === "object" && propertyValue !== null) {
            if (Array.isArray(propertyValue)) {
                // Handle arrays
                propertyValue.forEach((item, index) => {
                    if (typeof item === "object" && item !== null) {
                        Object.assign(subDocuments, getSubDocuments(item, `${fullKey}[${index}]`));
                    }
                });
            }
            else {
                // Handle nested objects
                Object.assign(subDocuments, getSubDocuments(propertyValue, fullKey));
            }
        }
    }
    return subDocuments;
}
export function clearSubDocumentProperties(instance) {
    for (const propertyKey of Object.keys(instance)) {
        if (isSubDocument(instance, propertyKey)) {
            const descriptor = Object.getOwnPropertyDescriptor(instance, propertyKey);
            if (Array.isArray(instance[propertyKey])) {
                instance[propertyKey] = [];
            }
            else if (descriptor === null || descriptor === void 0 ? void 0 : descriptor.configurable) {
                delete instance[propertyKey];
            }
            else {
                instance[propertyKey] = null;
            }
            continue; // Skip further processing for deleted property
        }
        const propertyValue = instance[propertyKey];
        // Check if the property is an object and not null
        if (typeof propertyValue === "object" && propertyValue !== null) {
            if (Array.isArray(propertyValue)) {
                // Handle arrays
                propertyValue.forEach((item) => {
                    if (typeof item === "object" && item !== null) {
                        clearSubDocumentProperties(item);
                    }
                });
            }
            else {
                // Handle nested objects
                clearSubDocumentProperties(propertyValue);
            }
        }
    }
}
export function parsePath(path) {
    const parts = [];
    const regex = /([^.[]+)|\[(\d+)\]/g;
    let match;
    while ((match = regex.exec(path)) !== null) {
        if (match[1]) {
            parts.push(match[1]);
        }
        else if (match[2]) {
            parts.push(parseInt(match[2], 10));
        }
    }
    return parts;
}
export function recordToISubDocument(values) {
    return Object.entries(values).map(([key, value]) => {
        return {
            key,
            value,
            rootDocRef: {},
        };
    });
}
// Sort paths to ensure that arrays are built in the correct order
function sortFunction(a, b) {
    const aParts = parsePath(a.key);
    const bParts = parsePath(b.key);
    const len = Math.min(aParts.length, bParts.length);
    for (let i = 0; i < len; i++) {
        const aPart = aParts[i];
        const bPart = bParts[i];
        if (aPart !== bPart) {
            if (typeof aPart === "number" && typeof bPart === "number") {
                return aPart - bPart;
            }
            return String(aPart).localeCompare(String(bPart));
        }
    }
    return aParts.length - bParts.length;
}
export function injectSubDocuments(instance, subDocuments) {
    // Extract paths and sort them to ensure array indices are processed in order
    const keyValuePair = subDocuments
        .map((subDoc) => {
        return { key: subDoc.key, value: subDoc.value };
    })
        .filter((kv) => Boolean(kv.key) && typeof kv.key === "string");
    keyValuePair.sort(sortFunction);
    for (const kv of keyValuePair) {
        const pathParts = parsePath(kv.key);
        setNestedSubProperty(instance, pathParts, kv.value);
    }
}
export function setNestedSubProperty(obj, pathParts, value) {
    const STOP_PATH = ["[object Object]", "undefined"];
    if (pathParts.length === 0)
        return;
    if (obj === null || obj === undefined) {
        console.error(`!!!   Error Object is not defined`, obj);
        return;
    }
    if (STOP_PATH.some((part) => pathParts.includes(part))) {
        console.error(`!!!  STOP Path detected`, pathParts, obj);
        return;
    }
    let current = obj;
    for (let i = 0; i < pathParts.length - 1; i++) {
        const path = pathParts[i];
        const nextPath = pathParts[i + 1];
        const nextIsArrayIndex = typeof nextPath === "number";
        if (current[path] === undefined || current[path] === null) {
            // Create an array or object depending on the next part
            try {
                current[path] = nextIsArrayIndex ? [] : {};
            }
            catch (e) {
                console.error(`!!! Error at create non existing <${path} ${pathParts.join(".")}> : ${nextIsArrayIndex ? "[]" : "{}"}`, current, obj);
                console.error(e);
            }
        }
        current = current[path];
    }
    const lastPart = pathParts[pathParts.length - 1];
    try {
        current[lastPart] = value;
    }
    catch (e) {
        console.error(`!!!   Error at SET value at : ${lastPart} ${pathParts.join(".")} =`, value, obj);
        console.error(e);
    }
}
// #endregion
export function stringifyAtFirebase(target, propertyKey) {
    // You can add custom logic here if needed
    // For simplicity, we'll just attach a metadata flag
    Reflect.metadata("stringifyAtFirebase", true)(target, propertyKey);
}
// Check if a property should be stringified at Firebase
export function shouldStringifyAtFirebase(target, propertyKey) {
    return (Reflect.getMetadata("stringifyAtFirebase", target, propertyKey) === true);
}
export function keepJsonStringAtFirebase(target, propertyKey) {
    // You can add custom logic here if needed
    // For simplicity, we'll just attach a metadata flag
    Reflect.metadata("keepJsonStringAtFirebase", true)(target, propertyKey);
}
// Check if a property should be stringified at Firebase
export function shouldKeepJsonStringAtFirebase(target, propertyKey) {
    return (Reflect.getMetadata("keepJsonStringAtFirebase", target, propertyKey) ===
        true);
}
export function ignoreAtFirebase(target, propertyKey) {
    Reflect.metadata("ignoreAtFirebase", true)(target, propertyKey);
}
export function shouldIgnoreAtFirebase(target, propertyKey) {
    return Reflect.getMetadata("ignoreAtFirebase", target, propertyKey) === true;
}
export function deepCopyWithoutIgnoreAtFirebase(obj) {
    if (obj === null || typeof obj !== "object") {
        return obj; // Return primitive values as is
    }
    if (Array.isArray(obj)) {
        return obj.map((item) => deepCopyWithoutIgnoreAtFirebase(item)); // Recursively copy array elements
    }
    if (typeof obj === "object") {
        const copiedObject = {};
        for (const key in obj) {
            if (!shouldIgnoreAtFirebase(obj, key)) {
                copiedObject[key] = deepCopyWithoutIgnoreAtFirebase(obj[key]); // Recursively copy object properties
            }
        }
        return copiedObject;
    }
    return obj;
}
export function JsonAtFirebase(dataObject) {
    if (dataObject === null || typeof dataObject !== "object") {
        return;
    }
    if (Array.isArray(dataObject)) {
        for (let i = 0; i < dataObject.length; i++) {
            JsonAtFirebase(dataObject[i]);
        }
    }
    if (typeof dataObject === "object") {
        const ignoreKeys = [];
        for (const key in dataObject) {
            if (shouldStringifyAtFirebase(dataObject, key) &&
                dataObject[key + "Json"]) {
                dataObject[key] = JSON.parse(dataObject[key + "Json"]);
                if (!shouldKeepJsonStringAtFirebase(dataObject, key))
                    ignoreKeys.push(key + "Json");
            }
            else if (!ignoreKeys.includes(key)) {
                JsonAtFirebase(dataObject[key]);
            }
        }
    }
}
export function parseJsonAtFirebase(obj) {
    if (obj === null || typeof obj !== "object") {
        return obj; // Return primitive values as is
    }
    if (Array.isArray(obj)) {
        return obj.map((item) => parseJsonAtFirebase(item)); // Recursively copy array elements
    }
    if (typeof obj === "object") {
        const copiedObject = {};
        const ignoreKeys = [];
        for (const key in obj) {
            if (shouldStringifyAtFirebase(obj, key) && obj[key + "Json"]) {
                const jsonString = obj[key + "Json"];
                try {
                    copiedObject[key] = JSON.parse(jsonString);
                }
                catch (e) {
                    console.error("!!!   Error parsing JSON", obj[key + "Json"]);
                    console.error(e);
                }
                if (!shouldKeepJsonStringAtFirebase(obj, key))
                    ignoreKeys.push(key + "Json");
            }
            else if (!ignoreKeys.includes(key)) {
                copiedObject[key] = parseJsonAtFirebase(obj[key]);
            }
        }
        return copiedObject;
    }
    return obj;
}
export function removeIgnoreDecorator(obj) {
    if (typeof obj !== "object" || obj === null) {
        return obj;
    }
    if (Array.isArray(obj)) {
        // If obj is an array, filter out elements with the decorator
        return obj.map((item) => removeIgnoreDecorator(item));
    }
    // Create a new object to store the filtered properties
    const newObj = {};
    for (const key in obj) {
        if (!shouldIgnoreAtFirebase(obj, key) &&
            !shouldStringifyAtFirebase(obj, key)) {
            // Recursively process non-ignored properties
            newObj[key] = removeIgnoreDecorator(obj[key]);
        }
        else if (shouldStringifyAtFirebase(obj, key)) {
            const json = JSON.stringify(obj[key]);
            newObj[key + "Json"] = json;
        }
    }
    return newObj;
}
function trimChar(originalString, charToRemove) {
    // Regex to match the character at the start and end of the string
    const regExp = new RegExp(`^${charToRemove}+|${charToRemove}+$`, "g");
    return originalString.replace(regExp, "");
}
//# sourceMappingURL=firebase.decorator.js.map