import admin from "firebase-admin";
import { ICollectionDocument } from "../interfaces/index.js";
import { CollectionDocument } from "../models/index.js";
/**
 * Asynchronously loads and injects sub-documents into a batch of root documents.
 *
 * This function takes an array of root documents, and for each document, it calls
 * `loadAndInjectSubDocuments` to load and inject sub-documents. It returns a promise
 * that resolves when all documents have been processed.
 *
 * @param {admin.firestore.Firestore} firestore - admin firestore.
 * @param {ICollectionDocument[]} rootDocs - An array of root documents into which sub-documents will be injected.
 * @returns {Promise<ICollectionDocument[]>} - A promise that resolves to an array of root documents, each injected with sub-documents.
 */
export declare function loadAndInjectSubDocumentsBatch(firestore: admin.firestore.Firestore, rootDocs: ICollectionDocument[]): Promise<ICollectionDocument[]>;
/**
 * Asynchronously loads and injects sub-documents into the given root document.
 *
 * This function takes a root document, checks if it has a valid document reference path,
 * and then attempts to load sub-documents from a Firestore sub-collection. If sub-documents
 * are found, they are injected into the root document.
 *
 * @template X - A type that extends ICollectionDocument.
 * @param {admin.firestore.Firestore} firestore - admin firestore.
 * @param {X} rootDoc - The root document into which sub-documents will be injected.
 * @returns {Promise<X>} - A promise that resolves to the root document, either injected with sub-documents or unchanged.
 */
export declare function loadAndInjectSubDocuments<X extends ICollectionDocument>(firestore: admin.firestore.Firestore, rootDoc: X): Promise<X>;
/**
 * Extracts sub-documents from a given document and saves them to Firestore.
 *
 * This function takes a document, extracts its sub-documents, and saves each sub-document
 * to a specified path in Firestore.
 *
 * @template T - A type that extends CollectionDocument.
 * @param {admin.firestore.Firestore} firestore - admin firestore.
 * @param {string} path - The Firestore path where the sub-documents will be saved.
 * @param {string} rootId - The ID of the root document.
 * @param {Partial<T>} document - The document from which sub-documents will be extracted.
 * @returns {Promise<void>} - A promise that resolves when all sub-documents have been saved.
 */
export declare function extractAndSaveSubDocuments<T extends CollectionDocument>(firestore: admin.firestore.Firestore, path: string, rootId: string, document: Partial<T>): Promise<void>;
//# sourceMappingURL=document.util.d.ts.map