var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { clearSubDocumentProperties, getSubDocuments, injectSubDocuments, } from "../decorators/firebase.decorator.js";
/**
 * Asynchronously loads and injects sub-documents into a batch of root documents.
 *
 * This function takes an array of root documents, and for each document, it calls
 * `loadAndInjectSubDocuments` to load and inject sub-documents. It returns a promise
 * that resolves when all documents have been processed.
 *
 * @param {admin.firestore.Firestore} firestore - admin firestore.
 * @param {ICollectionDocument[]} rootDocs - An array of root documents into which sub-documents will be injected.
 * @returns {Promise<ICollectionDocument[]>} - A promise that resolves to an array of root documents, each injected with sub-documents.
 */
export function loadAndInjectSubDocumentsBatch(firestore, rootDocs) {
    return __awaiter(this, void 0, void 0, function* () {
        const promises = rootDocs.map((rootDoc) => loadAndInjectSubDocuments(firestore, rootDoc));
        return Promise.all(promises);
    });
}
/**
 * Asynchronously loads and injects sub-documents into the given root document.
 *
 * This function takes a root document, checks if it has a valid document reference path,
 * and then attempts to load sub-documents from a Firestore sub-collection. If sub-documents
 * are found, they are injected into the root document.
 *
 * @template X - A type that extends ICollectionDocument.
 * @param {admin.firestore.Firestore} firestore - admin firestore.
 * @param {X} rootDoc - The root document into which sub-documents will be injected.
 * @returns {Promise<X>} - A promise that resolves to the root document, either injected with sub-documents or unchanged.
 */
export function loadAndInjectSubDocuments(firestore, rootDoc) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        if (!((_a = rootDoc === null || rootDoc === void 0 ? void 0 : rootDoc.documentReference) === null || _a === void 0 ? void 0 : _a.path)) {
            console.error("loadAndInjectSubDocuments: DocumentReference is missing");
            return rootDoc; // Return the original rootDoc if the path is missing
        }
        const rootDocRef = firestore.doc(rootDoc.documentReference.path);
        if (!rootDocRef) {
            return rootDoc; // Return the original rootDoc if Firestore reference is invalid
        }
        const subCollectionRef = rootDocRef.collection("sub");
        const subCollectionsSnap = yield subCollectionRef.get();
        const subDocuments = subCollectionsSnap.docs.map((y) => y.data());
        if (subDocuments && subDocuments.length > 0) {
            injectSubDocuments(rootDoc, subDocuments);
        }
        return rootDoc; // Return the rootDoc either injected or unchanged
    });
}
/**
 * Extracts sub-documents from a given document and saves them to Firestore.
 *
 * This function takes a document, extracts its sub-documents, and saves each sub-document
 * to a specified path in Firestore.
 *
 * @template T - A type that extends CollectionDocument.
 * @param {admin.firestore.Firestore} firestore - admin firestore.
 * @param {string} path - The Firestore path where the sub-documents will be saved.
 * @param {string} rootId - The ID of the root document.
 * @param {Partial<T>} document - The document from which sub-documents will be extracted.
 * @returns {Promise<void>} - A promise that resolves when all sub-documents have been saved.
 */
export function extractAndSaveSubDocuments(firestore, path, rootId, document) {
    return __awaiter(this, void 0, void 0, function* () {
        const subDocuments = getSubDocuments(document);
        const tasks = Object.keys(subDocuments).map((key) => {
            const ref = firestore.doc(`${path}/${rootId}/sub/${key}`);
            const subDoc = {
                key,
                value: subDocuments[key],
                rootDocRef: document.documentReference,
            };
            const subDocDTO = replaceUndefinedWithNull(subDoc);
            return ref.set(subDocDTO);
        });
        yield Promise.all(tasks);
        clearSubDocumentProperties(document);
    });
}
function replaceUndefinedWithNull(obj) {
    return JSON.parse(JSON.stringify(obj, (k, v) => {
        if (v === undefined) {
            return null;
        }
        return v;
    }));
}
//# sourceMappingURL=document.util.js.map