import { Timestamp } from "@firebase/firestore";
import type { ICollectionDocumentId } from "./ICollectionDocumentId.js";
import type { IDocumentReference } from "./IDocumentReference.js";
import type { IHasContainerRelationship } from "./IHasContainerRelationship.js";
import type { IHasParentChildRelationship } from "./IHasParentChildRelationship.js";
import type { IHasSiblingsRelationship } from "./IHasSiblingsRelationship.js";
import type { IHasUserCreataUpdate, IHasUserCreataUpdateAlgolia } from "./IHasUsers.js";
import { MetadataType } from "./IMetadata.js";
export interface ICollectionDocument extends ICollectionDocumentId, IHasUserCreataUpdate, IHasParentChildRelationship, IHasSiblingsRelationship, IHasContainerRelationship {
    workerLock?: ILock | null;
    notificationToken?: string | null;
    collectionPath?: string | null;
    documentReference?: IDocumentReference | null;
    sub?: {
        [key: string]: any;
    } | null;
    tags?: {
        [key: string]: boolean;
    } | null;
    metadata?: {
        [key: string]: MetadataType;
    } | null;
    backgroundTask?: IBackgroundTask | null;
    backgroundTasks?: IBackgroundTask[] | null;
    backgroundTaskLog?: IBackgroundTask[] | null;
    [key: string]: any;
}
export interface ICollectionDocumentAlgolia extends IHasUserCreataUpdateAlgolia {
    id: string;
    documentReference?: IDocumentReference | null;
    tags?: {
        [key: string]: boolean;
    } | null;
    metadata?: {
        [key: string]: MetadataType;
    } | null;
    notificationToken?: string | null;
}
export declare function sanitizeICollectionDocument(item: ICollectionDocument): boolean;
export declare function iCollectionDocumentToAlgolia(item: ICollectionDocument): ICollectionDocumentAlgolia;
export declare function iCollectionDocumentRequiredSubset(item: ICollectionDocument, collectionPath?: string): {
    id: string;
    createdBy: string | null | undefined;
    users: string[];
    groups: string[];
    groupClaims: any;
    documentReference: IDocumentReference;
};
export interface ILock {
    lockedAt?: Date | null;
    lockedBy?: string;
    message?: string;
}
export declare enum WorkerState {
    Idle = "Idle",
    Pending = "Pending",
    Running = "Running",
    Completed = "Completed",
    Failed = "Failed"
}
export declare enum WorkerAction {
    None = "None",
    VideoUploadedPipe = "videoUploadedPipe",
    NormalizeAudio = "NormalizeAudio",
    Transcode = "Transcode",
    Translate = "Translate",
    TranslateEditSpecification = "TranslateEditSpecification",
    Transcribe = "Transcribe",
    Render = "Render",
    AnalyzeVideo = "AnalyzeVideo",
    AnalyzeAudio = "AnalyzeAudio",
    AnalyzeImage = "AnalyzeImage"
}
export interface IBackgroundTask {
    id: string;
    name: string;
    action: WorkerAction;
    state: WorkerState;
    progress: number;
    pipeConfig: string;
    message?: string | null;
    createdAt: Date | Timestamp | null | undefined;
    updatedAt?: Date | Timestamp | null | undefined;
    finishedAt?: Date | Timestamp | null | undefined;
    traceReference?: string | null;
    traceReferences?: string[] | null;
}
export declare function getBackgroundTasks(actualData?: ICollectionDocument): IBackgroundTask[];
export declare function workerStateInProgress(state: WorkerState): boolean;
export declare function getTasksSorted(actualData: ICollectionDocument, asc: boolean): IBackgroundTask[] | undefined;
export declare function taskInProgress(data: ICollectionDocument): boolean;
export declare function getTasksInProgress(actualData?: ICollectionDocument): IBackgroundTask[];
/**
 * Retrieves a list of background tasks filtered by the specified action and sorts them.
 *
 * @param action - The action to filter tasks by.
 * @param actualData - Optional collection document containing the tasks.
 * @returns An array of background tasks filtered by the specified action and sorted.
 */
export declare function getTasksByAction(action: WorkerAction, actualData?: ICollectionDocument): IBackgroundTask[];
export declare function getTaskByAction(action: WorkerAction, actualData: ICollectionDocument, taskId?: string): IBackgroundTask | undefined;
/**
 * Retrieves the index of a task within the backgroundTasks array of the provided ICollectionDocument
 * that matches the specified WorkerAction.
 *
 * @param action - The action to search for within the backgroundTasks array.
 * @param actualData - The ICollectionDocument object containing the backgroundTasks array.
 *                     If null or undefined, the function returns -1.
 * @returns The index of the task that matches the specified action, or -1 if no match is found or if actualData is null/undefined.
 */
export declare function getTaskIndexByAction(action: WorkerAction, actualData?: ICollectionDocument | null | undefined, taskId?: string): number;
/**
 * Determines if the background tasks have changed between two ICollectionDocument instances.
 *
 * @param posterior - The posterior ICollectionDocument instance to compare.
 * @param prior - The prior ICollectionDocument instance to compare.
 * @param action - An optional WorkerAction to filter tasks by.
 * @returns A boolean indicating whether the background tasks have changed.
 */
export declare function backgroundTasksChanged(posterior?: ICollectionDocument, prior?: ICollectionDocument, action?: WorkerAction): boolean;
/**
 * Determines if a background task has changed state.
 *
 * This function compares the prior and posterior background tasks to check if there has been a change in their states.
 *
 * @param posteriorTask - The background task in its current state.
 * @param priorTask - The background task in its previous state.
 * @returns `true` if there is a change in the state of the background task, otherwise `false`.
 */
export declare function backgroundTaskChanged(posteriorTask?: IBackgroundTask, priorTask?: IBackgroundTask): boolean;
/**
 * Determines if any background task has been completed between two states of a collection document.
 *
 * @param posterior - The posterior state of the collection document.
 * @param prior - The prior state of the collection document.
 * @param action - (Optional) A specific worker action to filter tasks by.
 * @returns `true` if any background task has been completed; otherwise, `false`.
 */
export declare function anyBackgroundTaskCompleted(posterior?: ICollectionDocument, prior?: ICollectionDocument, action?: WorkerAction): boolean;
/**
 * Determines if a background task has been completed.
 *
 * @param posteriorTask - The task that is being checked for completion.
 * @param priorTask - The task that was previously being executed.
 * @returns `true` if the posterior task is completed and has changed from the prior task, otherwise `false`.
 */
export declare function backgroundTaskCompleted(posteriorTask?: IBackgroundTask, priorTask?: IBackgroundTask): boolean;
/**
 * Determines if there is a rising edge in background tasks between the prior and posterior states.
 *
 * @param action - The action to filter tasks by.
 * @param stateRise - The state that indicates a rising edge.
 * @param posterior - The current state of the collection document.
 * @param prior - The previous state of the collection document (optional).
 * @param blockingStates - An array of states that block the rising edge (optional).
 * @param compareId -  If true, the transition is only considered if it has the same id.
 * @returns `true` if a rising edge is detected, otherwise `false`.
 */
export declare function backgroundTasksRisingEdge(action: WorkerAction, stateRise: WorkerState, posterior: ICollectionDocument, prior?: ICollectionDocument, blockingStates?: WorkerState[], compareId?: boolean): IBackgroundTask | undefined;
/**
 * Determines if a background task has transitioned to a specified state (rising edge).
 *
 * @param action - The action associated with the worker.
 * @param stateRise - The state that signifies the rising edge.
 * @param current - The current background task.
 * @param previous - The previous background task.
 * @param blockingStates - An array of states that block the transition.
 * @param compareId - If true, the transition is only considered if it has the same id.
 * @returns `true` if the background task has transitioned to the specified state, otherwise `false`.
 */
export declare function backgroundTaskRisingEdge(action: WorkerAction, stateRise: WorkerState, current?: IBackgroundTask, previous?: IBackgroundTask, blockingStates?: WorkerState[], compareId?: boolean): boolean;
/**
 * Upserts a background task into the collection document.
 *
 * This function checks if the `backgroundTasks` array exists in the `actualData` object.
 * If it does not exist, it initializes it as an empty array.
 *
 * It then searches for an existing task in the `backgroundTasks` array that matches the
 * action of the provided `task` and has a state of `WorkerState.Pending`.
 *
 * If such a task is not found, it assigns a new ID to the task (if it doesn't already have one),
 * sets the creation date, and adds the task to the beginning of the `backgroundTasks` array.
 *
 * If a matching task is found, it updates the task in the array with the provided `task`.
 *
 * @param actualData - The collection document containing the background tasks.
 * @param task - The background task to be upserted.
 * @returns `true` if the task was added or updated, `false` otherwise.
 */
export declare function upsertBackgroundTask(actualData: ICollectionDocument, task: IBackgroundTask): boolean;
/**
 * Updates the state of a background task in the given document.
 *
 * This function can set the task to a specific state or advance it to the next state.
 * It updates the `updatedAt` timestamp and sets the `finishedAt` timestamp if the task is completed.
 * It can also update the task's message and trace reference.
 *
 * @param action - The action associated with the worker task.
 * @param doc - The document containing the background tasks.
 * @param options - Optional parameters:
 *   - taskId: The ID of the task to update. If not provided, the first task matching the action is updated.
 *   - newState: The new state to set for the task.
 *   - advanceToNextState: If true, advances the task to the next state.
 *   - message: An optional message to associate with the task.
 *   - traceRef: An optional trace reference to associate with the task.
 * @returns `true` if the task state was successfully updated, `false` otherwise.
 */
export declare function updateBackgroundTaskState(action: WorkerAction, doc: ICollectionDocument, options?: {
    taskId?: string;
    newState?: WorkerState;
    advanceToNextState?: boolean;
    message?: string | null;
    traceRef?: string | null;
}): boolean;
//# sourceMappingURL=ICollectionDocument.d.ts.map