/// <reference types="gl-matrix/index.js" />
import { vec2, mat3 } from "gl-matrix";
import { immerable } from "immer";
import { AnchorType, Aspect } from "../../enums/index.js";
import { IImageSettings } from "../../interfaces/index.js";
export type BoxData = {
    position: [number, number];
    size: [number, number];
    anchor: [number, number];
    ratio: [number, number];
    rotation: number;
    scalingEffect: Aspect;
    zIndex?: number;
};
/**
 * Represents a 2D box model with position, size, anchor properties.
 * The Coordinate system is based on x=center y=center of the box.
 * The interval of the local space is -1 to 1 with respect to the size of the box.
 * The box model can be nested within a parent box model, allowing for hierarchical transformations.
 *                         y
 *                         ↑
 *                         |
 *                    +----+----+
 *                    |    |    |
 *                    |    +----+-----→ x
 *                    |         |
 *                    +---------+
 **/
export declare class Box {
    [immerable]: boolean;
    static unitSize: [number, number];
    static originToTopLeft: [number, number];
    static topLeftToOrigin: [number, number];
    static originToTopRight: [number, number];
    static originToBottomLeft: [number, number];
    static originToBottomRight: [number, number];
    /**
     * Optional property representing the parent box model.
     *
     * @type {Box}
     */
    parent?: Box;
    /**
     * Represents the anchor point of the box model. it is the origin of the Position in it self.
     * It is in the box model's local space.
     *
     * @type {[number, number]}
     */
    anchor: [number, number];
    /**
     * The position of the box model represented as a 2D vector.
     * This property defines the coordinates of the box model in the the parent space.
     *
     * @type {[number, number]}
     */
    position: [number, number];
    get positionTranslationMatrix(): mat3;
    /**
     * The rotation of the box model in degrees around the anchor point CCW.
     *
     * @type {number}
     */
    rotation: number;
    get rotate(): boolean;
    get rotationMatrix(): mat3;
    /**
     * Represents the size of the box model in the parent space.
     *
     * @type {[number, number]}
     */
    size: [number, number];
    get scaleToParentSpaceMatrix(): mat3;
    /**
     * Optional property representing the ratio of the box.
     * It is a vector with two components (vec2) that can be used to define
     * the proportional relationship between the width and height of the box.
     *
     * @type {[number, number]}
     */
    ratio: [number, number];
    zIndex?: number;
    /**
     * Represents the scaling effect applied to the box model.
     *
     * @type {Aspect}
     * @default Aspect.None
     */
    scalingEffect: Aspect;
    get sizeInParentSpace(): vec2;
    constructor(options?: {
        parent?: Box;
        size?: [number, number] | vec2;
        position?: [number, number] | vec2;
        anchor?: [number, number] | vec2;
        ratio?: [number, number] | vec2;
        rotation?: number;
        zIndex?: number;
        scalingEffect?: Aspect;
    });
    /**
     * Scales a given position vector from local space (-0.5 to 0.5) to parent space.
     *
     * @param A - The position vector in local space.
     * @returns The scaled position vector in parent space.
     */
    scaleToParentSpace(A: vec2): vec2;
    /**
     * Translates a given vector to the parent's coordinate system.
     *
     * @param A - The vector to be translated.
     * @returns The translated vector in the parent's coordinate system.
     */
    translateToParent(A: vec2): vec2;
    /**
     * Rotates a given vector `A` to the parent's coordinate system based on the current object's rotation and position.
     *
     * @param A - The vector to be rotated.
     * @returns The rotated vector in the parent's coordinate system.
     */
    rotateToParent(A: vec2): vec2;
    /**
     * Projects a given point `A` to the parent space of the current box model.
     *
     * This method performs the following transformations in sequence:
     * 1. Translates the point `A` with respect to the anchor of the current box.
     * 2. Scales the translated point to the parent space.
     * 3. Translates the scaled point to the parent space.
     * 4. Rotates the translated point to the parent space.
     *
     * @param A - The point to be projected, represented as a `vec2`.
     * @returns The projected point in the parent space, represented as a `vec2`.
     */
    projectToParentSpace(A: vec2): vec2;
    /**
     * Adjusts the size of the current box to fit within its parent's dimensions while maintaining the aspect ratio.
     *
     * This method scales the box's size based on the smaller scaling factor between the width and height ratios.
     * If the parent or ratio properties are not defined, the method will return without making any changes.
     *
     * @remarks
     * - The `parent` property should have a `size` attribute representing the dimensions of the parent container.
     * - The `ratio` property should be a tuple representing the aspect ratio of the box.
     * - The `size` property will be scaled in place to fit within the parent's dimensions.
     */
    fit(): void;
    /**
     * Adjusts the size of the box to fill its parent container while maintaining the aspect ratio.
     *
     * This method scales the box's size based on the ratio and the size of its parent container.
     * It ensures that the box fills the parent container as much as possible without distorting
     * the aspect ratio.
     *
     * @remarks
     * - If the box does not have a parent or a defined ratio, the method returns immediately.
     * - The scaling factor is determined by the larger of the two ratios (width or height).
     */
    fill(): void;
    /**
     * Converts the current Box instance to a BoxData object.
     *
     * @returns {BoxData} The BoxData representation of the current Box instance.
     */
    toData(): BoxData;
    /**
     * Inverts the Y-coordinate of a given 2D vector.
     *
     * @param A - The input vector to be inverted.
     * @returns A new vector with the Y-coordinate inverted.
     */
    static invertY(A: vec2): vec2;
    /**
     * Converts a vector from a center-origin coordinate system to a top-left-origin coordinate system.
     *
     * @param A - The vector in the center-origin coordinate system.
     * @returns The vector in the top-left-origin coordinate system.
     */
    static toPixelCoordinateSystem(A: vec2): vec2;
    /**
     * Creates and returns a new instance of the `Box` class.
     *
     * @returns {Box} A new `Box` instance.
     */
    static default(): Box;
    /**
     * Creates a new `Box` instance from the provided data.
     *
     * @param data - The data to create the `Box` from.
     * @param parent - An optional parent `Box`.
     * @returns A new `Box` instance.
     */
    static fromData(data: BoxData, parent?: Box): Box;
    /**
     * Creates a new `Box` instance from the provided image settings.
     *
     * @param settings - The image settings to create the `Box` from.
     * @param parent - Optional parent image settings to create a parent `Box`.
     * @param scalingEffect - Optional scaling effect to apply. Defaults to `Aspect.Fit`.
     * @returns A new `Box` instance configured with the provided settings.
     */
    static fromIImageSettings(settings: IImageSettings, parent?: IImageSettings, scalingEffect?: Aspect): Box;
}
/**
 * Reduces a fraction to its simplest form.
 *
 * @param numerator - The numerator of the fraction. If not provided, defaults to 0.
 * @param denominator - The denominator of the fraction. If not provided, defaults to 1.
 * @returns A tuple containing the simplified numerator and denominator.
 */
export declare function reduceFraction(numerator?: number, denominator?: number): [number, number];
/**
 * Calculates the greatest common divisor (GCD) of two numbers using the Euclidean algorithm.
 *
 * @param a - The first number.
 * @param b - The second number.
 * @returns The greatest common divisor of the two numbers.
 */
export declare function greatestCommonDivisor(a: number, b: number): number;
export declare function parseAnchorType(anchorX: AnchorType, anchorY: AnchorType): [number, number];
export declare function toAnchorType(A: [number, number]): [AnchorType, AnchorType];
//# sourceMappingURL=box.d.ts.map