import { describe, it, expect, beforeEach } from "vitest";
import { spec, specWithSub } from "../data";
import { DEFAULT_VIDEO_SETTINGS } from "../interfaces/index.js";
import { PixelFormat, ProResProfile, VideoCodecType, VideoFileType, VideoImageFormatType, VideoRatioType, VideoResolution, } from "../enums/index.js";
import { EditSpecification } from "./editSpecification.js";
describe("EditSpecification", () => {
    let specString;
    let specWithSubString;
    const VIDEO_SETTINGS = {
        frameRate: 30,
        durationInFrames: 5,
        videoCodec: VideoCodecType.ProRes,
        videoImageFormat: VideoImageFormatType.JPEG,
        pixelFormat: PixelFormat.yuv420p10le,
        proresProfile: ProResProfile.ProRes444XQ,
        fileType: VideoFileType.mp4,
        isLandscape: true,
        resolution: VideoResolution.HD,
        ratio: VideoRatioType.R1To1,
        ratioFraction: [1, 1],
        width: 720,
        height: 720,
        alphaChannel: false,
    };
    beforeEach(() => {
        specString = JSON.stringify(spec);
        specWithSubString = JSON.stringify(specWithSub);
    });
    describe("EditSpecification Serialize", () => {
        it("should serialize and deserialize to and from JSON", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            const reserializedString = spec1.toJSON();
            const hash1 = spec1.getMd5();
            const spec2 = EditSpecification.fromJSON(reserializedString);
            const hash2 = spec2.getMd5();
            const equal = spec1.equals(spec2);
            expect(spec1).toBeInstanceOf(EditSpecification);
            expect(spec2).toBeInstanceOf(EditSpecification);
            expect(hash1).toEqual(hash2);
            expect(equal).toEqual(true);
        });
    });
    describe("Getter", () => {
        it("should return libraryItems string[]", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            const libItems = spec1.libraryItems;
            expect(libItems).containSubset([
                "24903b60-5bd5-4f49-a587-dacefb847cd8",
                "e3eef905-6525-46eb-bb50-4c2b5f1cf79c",
            ]);
        });
        it("should return comparable Clips", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            const { comparableClips } = spec1;
            expect(comparableClips.length).toBe(1);
        });
    });
    describe("Setter", () => {
        it("should ", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            const prior = Object.assign({}, spec1.videoSettings);
            spec1.videoSettings = undefined;
            expect(spec1.videoSettings).toEqual(prior);
        });
    });
    describe("setLanguage", () => {
        it("should change the language of all layers", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            spec1.setLanguage("test");
            expect(spec1.clips[0].layers[0].language).toEqual("test");
            expect(spec1.clips[0].layers[1].language).toEqual("test");
        });
    });
    describe("setVideoSettings", () => {
        it("should change the videoSettings to DEFAULT", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            spec1.setVideoSettings(DEFAULT_VIDEO_SETTINGS);
            expect(spec1.width).toEqual(DEFAULT_VIDEO_SETTINGS.width);
            expect(spec1.height).toEqual(DEFAULT_VIDEO_SETTINGS.height);
            expect(spec1.fps).toEqual(DEFAULT_VIDEO_SETTINGS.frameRate);
            expect(spec1.durationInFrames).toEqual(DEFAULT_VIDEO_SETTINGS.durationInFrames);
            expect(spec1.videoSettings).toEqual(DEFAULT_VIDEO_SETTINGS);
        });
        it("should change the videoSettings to VIDEO_SETTINGS", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            spec1.setVideoSettings(VIDEO_SETTINGS);
            expect(spec1.width).toEqual(VIDEO_SETTINGS.width);
            expect(spec1.height).toEqual(VIDEO_SETTINGS.height);
            expect(spec1.fps).toEqual(VIDEO_SETTINGS.frameRate);
            expect(spec1.durationInFrames).toEqual(VIDEO_SETTINGS.durationInFrames);
            expect(spec1.videoSettings).toEqual(VIDEO_SETTINGS);
        });
    });
    describe("Compare", () => {
        it("spec1 should equals spec2", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            const spec2 = EditSpecification.fromJSON(specWithSubString);
            const equal = spec1.equals(spec2);
            expect(spec1).toBeInstanceOf(EditSpecification);
            expect(spec2).toBeInstanceOf(EditSpecification);
            expect(equal).toEqual(true);
        });
        it("spec1 should not equals spec2", () => {
            const spec1 = EditSpecification.fromJSON(specString);
            const spec2 = EditSpecification.fromJSON(specWithSubString);
            const equal = spec1.equals(spec2);
            expect(spec1).toBeInstanceOf(EditSpecification);
            expect(spec2).toBeInstanceOf(EditSpecification);
            expect(equal).toEqual(false);
        });
        it("spec1 should not equals spec2 - text changed", () => {
            const spec1 = EditSpecification.fromJSON(specWithSubString);
            const spec2 = EditSpecification.fromJSON(specWithSubString);
            spec2.clips[0].layers[0].text = "different";
            const equal = spec1.equals(spec2);
            expect(spec1).toBeInstanceOf(EditSpecification);
            expect(spec2).toBeInstanceOf(EditSpecification);
            expect(equal).toEqual(false);
        });
    });
});
//# sourceMappingURL=editSpecification.spec.js.map