var _a;
import { immerable } from "immer";
import { Anchor, TimingAction } from "../enums/index.js";
import { clamp } from "../extensions/index.js";
const MAX_VALUE = 100000000000;
export class FrameRange {
    // Set the callback and arguments for the conditional method
    use(callback, ...args) {
        const newInstance = this.clone();
        newInstance.callback = callback;
        newInstance.args = args;
        return newInstance;
    }
    // Execute the conditional method
    execute(condition = true) {
        if (!condition)
            return this;
        const newInstance = this.clone();
        if (newInstance.callback) {
            newInstance.callback(...newInstance.args);
            newInstance.callback = null;
        }
        return newInstance;
    }
    get center() {
        return Math.round((this.start + this.end) / 2);
    }
    get duration() {
        return this.end - this.start;
    }
    set duration(value) {
        this.end = this.start + clamp(value, 1, MAX_VALUE);
    }
    // eslint-disable-next-line accessor-pairs
    set durationFromEnd(value) {
        this.start = this.end - clamp(value, 1, MAX_VALUE);
    }
    static fromSeconds(start, end, fps, anchor) {
        return new FrameRange(millisecondsToFps(Math.round(start * 1000), fps), millisecondsToFps(Math.round(end * 1000), fps), anchor);
    }
    static fromMilliseconds(start, end, fps, anchor) {
        return new FrameRange(millisecondsToFps(start, fps), millisecondsToFps(end, fps), anchor);
    }
    static fromIFrameRange(item) {
        return new FrameRange(item.start, item.end, item.anchor);
    }
    static get positive() {
        return new FrameRange(0, MAX_VALUE);
    }
    static get negative() {
        return new FrameRange(-MAX_VALUE, 0);
    }
    static get zero() {
        return new FrameRange(0, 0);
    }
    static get max() {
        return new FrameRange(-MAX_VALUE, MAX_VALUE);
    }
    static get minClipRange() {
        return new FrameRange(0, 4);
    }
    constructor(start, end, anchor, zeroLength) {
        this[_a] = true; // Option 1
        // #region FluentApi
        this.callback = null;
        this.args = [];
        // #endregion FluentApi
        this.start = 0;
        this.end = 1;
        this.anchor = null;
        this.toString = () => {
            return `frRng: ${this.start} -> ${this.end}  [${this.duration}]`;
        };
        this.start = start !== null && start !== void 0 ? start : 0;
        this.anchor = anchor !== null && anchor !== void 0 ? anchor : null;
        if ((end || end === 0) && end > this.start) {
            this.end = end;
        }
        else if (zeroLength) {
            this.end = this.start;
        }
        else
            this.end = this.start + 1;
    }
    // #region get-Subsets
    aAndNotB(b) {
        const differences = FrameRange.differences(this, [b]);
        return differences;
    }
    aIntersectB(b) {
        const intersection = this.getIntersection(b);
        if (!intersection)
            return FrameRange.empty();
        return intersection;
    }
    aXorB(b) {
        const differencesA = FrameRange.differences(this, [b]);
        const differencesB = FrameRange.differences(b, [this]);
        return [...differencesA, ...differencesB];
    }
    union(b) {
        if (Array.isArray(b)) {
            const result = FrameRange.unionOfAll([this, ...b]);
            return result;
        }
        if (b instanceof FrameRange) {
            if (this.isIntersecting(b)) {
                const minStart = Math.min(this.start, b.start);
                const maxEnd = Math.max(this.end, b.end);
                return [new FrameRange(minStart, maxEnd)];
            }
            return [this.clone(), b.clone()];
        }
        throw new Error(`FrameRange.union unreachable code`);
    }
    static unionOfAll(ranges) {
        let result = [];
        for (let i = 0; i < ranges.length; i++) {
            let unionWithOthers = [ranges[i]];
            for (let j = 0; j < ranges.length; j++) {
                if (i !== j) {
                    let temp = [];
                    unionWithOthers.forEach((range) => {
                        temp = [...temp, ...range.union(ranges[j])];
                    });
                    unionWithOthers = temp;
                }
            }
            result = [...result, ...unionWithOthers];
        }
        // Remove duplicates
        result = result.filter((range, index, self) => index === self.findIndex((t) => t.isEqual(range)));
        // Remove ranges that are inside other ranges
        result = result.filter((range) => {
            return !result.find((x) => range.isInside(x) && !x.isEqual(range));
        });
        return result;
    }
    combineFrameRanges(ranges) {
        let combined = [];
        // Sort the ranges by their start value
        ranges.sort((a, b) => a.start - b.start);
        combined.push(ranges[0]);
        for (let i = 1; i < ranges.length; i++) {
            const newCombined = [];
            let merged = false;
            for (let j = 0; j < combined.length; j++) {
                const result = combined[j].union(ranges[i]);
                if (result.length === 1) {
                    newCombined.push(result[0]);
                    merged = true;
                }
                else {
                    newCombined.push(combined[j]);
                }
            }
            if (!merged) {
                newCombined.push(ranges[i]);
            }
            combined = newCombined;
        }
        return combined;
    }
    // #endregion get-Subsets
    // #region get-Derived
    toIFrameRange() {
        var _b;
        return {
            start: this.start,
            end: this.end,
            anchor: (_b = this.anchor) !== null && _b !== void 0 ? _b : null,
        };
    }
    withPadding(paddingStart, paddingEnd = null, zeroLength = undefined) {
        if (this.duration <= 1)
            return this.clone();
        if (!paddingStart)
            paddingStart = 0;
        if (!paddingEnd)
            paddingEnd = paddingStart;
        if (paddingStart + paddingEnd > this.duration) {
            paddingStart = this.duration / 4;
            paddingEnd = this.duration / 4;
        }
        const shiftedStart = this.start + paddingStart;
        const shiftedEnd = this.end - paddingEnd;
        return new FrameRange(Math.min(shiftedStart, shiftedEnd, this.end), Math.max(shiftedStart, shiftedEnd, this.start), this.anchor, zeroLength);
    }
    withDurationFromStart(duration, limit = true) {
        const end = limit
            ? Math.min(this.start + duration, this.end)
            : this.start + duration;
        return new FrameRange(this.start, end, this.anchor);
    }
    scale(factor) {
        // Limit factor to positive numbers > Number.EPSILON
        factor = Math.max(factor, Number.EPSILON);
        return new FrameRange(this.start * factor, this.end * factor, this.anchor);
    }
    scaleDuration(scale) {
        switch (this.anchor) {
            case Anchor.Center:
                return this.scaleDurationFromCenter(scale);
            case Anchor.End:
                return this.scaleDurationFromEnd(scale);
            default:
                return this.scaleDurationFromStart(scale);
        }
    }
    scaleDurationFromStart(factor) {
        factor = Math.max(factor, Number.EPSILON);
        return FrameRange.fromStart(this.start, this.duration * factor, this.anchor);
    }
    scaleDurationFromCenter(factor) {
        factor = Math.max(factor, Number.EPSILON);
        const center = (this.start + this.end) / 2;
        const halfDuration = this.duration / 2;
        const newHalfDuration = halfDuration * factor;
        const newStart = center - newHalfDuration;
        const newEnd = center + newHalfDuration;
        return new FrameRange(newStart, newEnd, this.anchor);
    }
    scaleDurationFromEnd(factor) {
        factor = Math.max(factor, Number.EPSILON);
        return FrameRange.fromEnd(this.end, this.duration * factor, this.anchor);
    }
    shiftLeft(value) {
        return this.add(-value);
    }
    shiftRight(value) {
        return this.add(value);
    }
    trimmLeft(value) {
        return this.addStart(value);
    }
    trimmRight(value) {
        return this.subtractEnd(value);
    }
    add(value, minStart, maxEnd, action) {
        const result = new FrameRange(this.start + value, this.end + value, this.anchor);
        if (minStart || maxEnd) {
            const limit = new FrameRange(minStart !== null && minStart !== void 0 ? minStart : 0, maxEnd !== null && maxEnd !== void 0 ? maxEnd : MAX_VALUE, this.anchor);
            return result.pushInside(limit, action !== null && action !== void 0 ? action : TimingAction.Trim);
        }
        return result;
    }
    addStart(value) {
        const limitedDelta = Math.min(value, this.duration - 1);
        return new FrameRange(this.start + limitedDelta, this.end, this.anchor);
    }
    addEnd(value) {
        const limitedEnd = Math.max(this.start + 1, this.end + value);
        return new FrameRange(this.start, limitedEnd, this.anchor);
    }
    addRelative(relative, timing = TimingAction.Trim) {
        const start = this.start + relative.start;
        const end = timing === TimingAction.Expand
            ? start + relative.duration
            : Math.min(start + relative.duration, this.end);
        return new FrameRange(start, end, this.anchor);
    }
    subtract(value) {
        return new FrameRange(this.start - value, this.end - value, this.anchor);
    }
    subtractStart(value) {
        return new FrameRange(this.start - value, this.end, this.anchor);
    }
    subtractEnd(value) {
        const limitedDelta = Math.min(value, this.duration - 1);
        return new FrameRange(this.start, this.end - limitedDelta, this.anchor);
    }
    split() {
        const center = Math.round((this.start + this.end) / 2);
        const r1 = new FrameRange(this.start, center);
        const r2 = new FrameRange(center, this.end);
        return { first: r1, last: r2 };
    }
    withDurationFromEnd(duration) {
        const limitedDuration = Math.max(duration, 1);
        return new FrameRange(Math.min(Math.max(this.end - limitedDuration, this.start), this.end - 1), this.end, this.anchor);
    }
    withAnchor(anchor) {
        if (!anchor)
            return this.clone();
        return new FrameRange(this.start, this.end, anchor);
    }
    clearAnchor() {
        return new FrameRange(this.start, this.end, null);
    }
    getFadeIn(fadeDuration, padding = 0) {
        const range = this.withPadding(padding);
        const innerRange = range.withPadding(fadeDuration);
        return [range.start, innerRange.start];
    }
    getFadeOut(fadeDuration, padding = 0) {
        const range = this.withPadding(null, padding);
        const innerRange = range.withPadding(null, fadeDuration);
        return [innerRange.end, range.end];
    }
    getFadeInOut(fadeDuration, padding = 0) {
        const range = this.withPadding(padding);
        const innerRange = range.withPadding(fadeDuration);
        return [range.start, innerRange.start, innerRange.end, range.end];
    }
    getFadeInOutAtStart(fadeDuration, visibleDuration, padding = 0) {
        if (fadeDuration < 2)
            throw new Error(`fadeDuration must be at least 2 frame`);
        if (visibleDuration < 2)
            throw new Error(`visibleDuration must be at least 2 frame`);
        const range = this.withPadding(padding);
        const rangeOfFade = range.withDurationFromStart(visibleDuration);
        const innerRange = rangeOfFade.withPadding(fadeDuration, fadeDuration, true);
        return [
            rangeOfFade.start,
            innerRange.start,
            innerRange.end,
            rangeOfFade.end,
        ];
    }
    getFadeInOutAtEnd(fadeDuration, visibleDuration, padding = 0) {
        if (fadeDuration < 2)
            throw new Error(`fadeDuration must be at least 2 frame`);
        if (visibleDuration < 2)
            throw new Error(`visibleDuration must be at least 2 frame`);
        const range = this.withPadding(padding);
        const rangeOfFade = range.withDurationFromEnd(visibleDuration);
        const innerRange = rangeOfFade.withPadding(fadeDuration, fadeDuration, true);
        return [
            rangeOfFade.start,
            innerRange.start,
            innerRange.end,
            rangeOfFade.end,
        ];
    }
    getFadeInOutWithOther(other) {
        return [this.start, this.end, other.start, other.end];
    }
    getFade() {
        return [this.start, this.end];
    }
    getCutCenter() {
        const centerThis = Math.floor((this.start + this.end) / 2);
        return [centerThis, centerThis + 1];
    }
    getCutCenterInOutWithOther(other) {
        const centerThis = this.getCutCenter();
        const centerOther = other.getCutCenter();
        if (centerThis[0] >= centerOther[0])
            throw new Error(`other FrameRange must be bigger then this ${this} ${other}`);
        return [...centerThis, ...centerOther];
    }
    getRelativeFrame(frame) {
        return frame - this.start;
    }
    getRelativeFrameClamp(frame) {
        const relative = this.getRelativeFrame(frame);
        return clamp(relative, 0, this.duration);
    }
    startAt(start = 0) {
        return new FrameRange(start, start + this.duration);
    }
    endDuration(duration) {
        if (duration < 0)
            throw new Error(`duration must be positive`);
        this.duration = duration;
    }
    calculateNormPosition(frame) {
        const norm = this.getRelativeFrame(frame) / this.duration;
        // Return a number between 0 and 1 and clamp it
        return clamp(norm, 0, 1);
    }
    getIntersection(other) {
        if (this.isIntersecting(other)) {
            const intersectionStart = Math.max(this.start, other.start);
            const intersectionEnd = Math.min(this.end, other.end);
            return new FrameRange(intersectionStart, intersectionEnd);
        }
        return null;
    }
    // #endregion get-Derived
    // #region compare
    isEmpty() {
        return this.start === 0 && this.end === 0;
    }
    isDefault() {
        return this.start === 0 && this.end === 1;
    }
    isValid() {
        if (this.start === null || this.start === undefined)
            return false;
        if (this.end === null || this.end === undefined)
            return false;
        if (!Number.isFinite(this.start) || !Number.isFinite(this.end))
            return false;
        return this.start < this.end;
    }
    isIntersecting(other) {
        if (!this.isValid())
            throw new Error(`self frames are not valid`);
        if (!other || !other.isValid())
            throw new Error(`other frames are not valid`);
        return this.start < other.end && other.start < this.end;
    }
    isBefore(other) {
        if (!this.isValid())
            throw new Error(`self frames are not valid`);
        if (!other || !other.isValid())
            throw new Error(`other frames are not valid`);
        return this.end <= other.start;
    }
    isBehind(other) {
        if (!this.isValid())
            throw new Error(`self frames are not valid`);
        if (!other || !other.isValid())
            throw new Error(`other frames are not valid`);
        return this.start >= other.end;
    }
    isInside(other) {
        if (other === undefined)
            return false;
        if (!this.isValid())
            throw new Error(`self frames are not valid ${this}`);
        if (!other || !other.isValid())
            throw new Error(`other frames are not valid`);
        const result = other.start <= this.start && this.end <= other.end;
        return result;
    }
    startIsEqual(other) {
        return this.start === (other === null || other === void 0 ? void 0 : other.start);
    }
    endIsEqual(other) {
        return this.end === (other === null || other === void 0 ? void 0 : other.end);
    }
    isEqual(other) {
        return (this.start === (other === null || other === void 0 ? void 0 : other.start) &&
            this.end === (other === null || other === void 0 ? void 0 : other.end) &&
            this.anchor === (other === null || other === void 0 ? void 0 : other.anchor));
    }
    containsFrame(frame) {
        if (!this.isValid())
            throw new Error(`self frames are not valid ${this.start} <= ${this.end}`);
        const result = this.start <= frame && frame <= this.end;
        return result;
    }
    isInDurationRange(min, max) {
        const minOk = min ? min <= this.duration : true;
        const maxOk = max ? this.duration <= max : true;
        return minOk && maxOk;
    }
    deltaStart(other) {
        return this.start - other.start;
    }
    deltaEnd(other) {
        return this.end - other.end;
    }
    // #endregion compare
    // #region edit
    /// <summary>
    /// Returns the FrameRange with the padding added to the start and end.
    /// positive padding will make the FrameRange shorter
    /// </summary>
    addPadding(padding, paddingEnd = null) {
        if (this.duration > padding) {
            return new FrameRange(this.start + padding, this.end - (paddingEnd !== null && paddingEnd !== void 0 ? paddingEnd : padding), this.anchor);
        }
        return this;
    }
    addPaddingLeft(padding) {
        if (this.duration > padding) {
            return new FrameRange(this.start + padding, this.end, this.anchor);
        }
        return this;
    }
    tryAddPaddingLeft(padding) {
        if (this.duration > padding) {
            return new FrameRange(this.start + padding, this.end, this.anchor);
        }
        return undefined;
    }
    addPaddingRight(padding) {
        if (this.duration > padding) {
            return new FrameRange(this.start, this.end - padding, this.anchor);
        }
        return this;
    }
    tryAddPaddingRight(padding) {
        if (this.duration > padding) {
            return new FrameRange(this.start, this.end - padding, this.anchor);
        }
        return undefined;
    }
    round() {
        return new FrameRange(Math.round(this.start), Math.round(this.end), this.anchor);
    }
    pushOtherBehind(other, padding = 0) {
        const selfWithPadding = this.addPadding(padding);
        if (selfWithPadding.isIntersecting(other) ||
            other.isBefore(selfWithPadding)) {
            const newStart = selfWithPadding.end;
            const newEnd = newStart + other.duration;
            return new FrameRange(newStart, newEnd, this.anchor);
        }
        return other;
    }
    pushOtherInFront(other, padding = 0) {
        const selfWithPadding = this.addPadding(padding);
        if (selfWithPadding.isIntersecting(other) ||
            other.isBehind(selfWithPadding)) {
            const newStart = selfWithPadding.start - other.duration;
            const newEnd = selfWithPadding.start;
            return new FrameRange(newStart, newEnd, other.anchor);
        }
        return other;
    }
    limitDuration(maxDuration) {
        if (maxDuration && this.duration > maxDuration) {
            return this.withDurationFromStart(maxDuration, false);
        }
        return this;
    }
    pushInside(other, action = TimingAction.Shift) {
        const trimm = action === TimingAction.Trim;
        if (this.isInside(other))
            return this.clone();
        if (this.duration > other.duration)
            return other.withAnchor(this.anchor);
        const startInsideOther = this.start < other.end && this.start > other.start;
        const endInsideOther = this.end > other.start && this.end < other.end;
        if (startInsideOther || this.isBehind(other)) {
            const newStart = trimm ? this.start : other.end - this.duration;
            const newEnd = other.end;
            const insideOther = new FrameRange(newStart, newEnd, this.anchor);
            return insideOther.withAnchor(this.anchor);
        }
        if (endInsideOther || this.isBefore(other)) {
            const newStart = other.start;
            const newEnd = trimm ? this.end : other.start + this.duration;
            const insideOther = new FrameRange(newStart, newEnd, this.anchor);
            return insideOther.withAnchor(this.anchor);
        }
        throw new Error(`pushInside code should not reach this code far`);
    }
    clamp(other) {
        return this.pushInside(other);
    }
    pushApart(other) {
        if (other)
            return this.expand(other);
        return this.clone();
    }
    expand(other, padding = 0) {
        const otherWithPadding = other.addPadding(padding);
        if (otherWithPadding.isInside(this)) {
            return this;
        }
        return new FrameRange(Math.min(this.start, otherWithPadding.start), Math.max(this.end, otherWithPadding.end), this.anchor);
    }
    // #endregion edit
    clone() {
        return new FrameRange(0 + this.start, 0 + this.end, this.anchor);
    }
    static compareStartAndEnd(a, b) {
        // First compare the start values
        if (a.start < b.start) {
            return -1;
        }
        if (a.start > b.start) {
            return 1;
        }
        // If start values are equal, then compare the end values
        if (a.end < b.end) {
            return -1;
        }
        if (a.end > b.end) {
            return 1;
        }
        // If both start and end values are equal, return 0
        return 0;
    }
    static compareStart(a, b) {
        if (a.start < b.start) {
            return -1;
        }
        if (a.start > b.start) {
            return 1;
        }
        return 0;
    }
    static compareEnd(a, b) {
        if (a.end < b.end) {
            return -1;
        }
        if (a.end > b.end) {
            return 1;
        }
        return 0;
    }
    static compareDuration(a, b) {
        if (a.duration < b.duration) {
            return -1;
        }
        if (a.duration > b.duration) {
            return 1;
        }
        return 0;
    }
    static combineRelativeAndCut(relativeRange, cutRange) {
        if (cutRange === undefined || cutRange === null)
            return relativeRange.clone();
        return relativeRange.clone();
    }
    static empty() {
        return new FrameRange(0, 0, null, true);
    }
    static fromStart(startFrame, duration, anchor) {
        return new FrameRange(0 + startFrame, startFrame + duration, anchor);
    }
    static fromCenter(center, duration, anchor) {
        const halfDuration = duration / 2;
        return new FrameRange(Math.floor(center - halfDuration), Math.ceil(center + halfDuration), anchor);
    }
    static fromEnd(lastFrame, duration, anchor) {
        return new FrameRange(lastFrame - duration, lastFrame, anchor);
    }
    static first(frames) {
        if (frames.length === 0)
            return null;
        const sorted = frames.sort(FrameRange.compareStart);
        return sorted[0];
    }
    static last(frames) {
        if (frames.length === 0)
            return null;
        const sorted = frames.sort(FrameRange.compareStart).reverse();
        return sorted[0];
    }
    static withStart(frames, start) {
        var _b;
        if (frames.length === 0)
            return null;
        const sorted = frames.sort(FrameRange.compareStart);
        return (_b = sorted.find((x) => x.start === start)) !== null && _b !== void 0 ? _b : null;
    }
    static withEnd(frames, end) {
        var _b;
        if (frames.length === 0)
            return null;
        const sorted = frames.sort(FrameRange.compareStart);
        return (_b = sorted.find((x) => x.end === end)) !== null && _b !== void 0 ? _b : null;
    }
    static areEqual(a, b) {
        if (!a || !b)
            return false;
        return a.isEqual(b);
    }
    static arrayEquals(a, b) {
        if (a.length !== b.length)
            return false;
        // Sort the arrays
        a.sort(FrameRange.compareStartAndEnd);
        b.sort(FrameRange.compareStartAndEnd);
        for (let index = 0; index < a.length; index++) {
            const aItem = a[index];
            const bItem = b[index];
            if (!aItem.isEqual(bItem))
                return false;
        }
        return true;
    }
    // #region SET[]
    /**
     * A \ B (also written A − B[]) is the set of all things that belong to A but not B[]
     */
    static differences(a, bItems) {
        const differences = FrameRange.differencesRecursive(a, bItems);
        const distinctFixed = differences.filter((n, i) => {
            return (differences.findIndex((x) => x.isEqual(n)) === i &&
                !differences.find((x) => x.isInside(n) && !x.isEqual(n)));
        });
        return distinctFixed;
    }
    static differencesRecursive(a, bItems) {
        // Const differences: FrameRange[] = [];
        const bSubItems = FrameRange.distinctFrameRanges(bItems);
        if (!bSubItems || bSubItems.length === 0)
            return [a];
        let newAs = [a.clone()];
        for (const b of bSubItems) {
            const res = [...newAs.map((x) => FrameRange.difference(x, b)).flat()];
            newAs = res;
        }
        return newAs;
    }
    /**
     * A \ B (also written A − B) is the set of all things that belong to A but not B
     */
    static difference(a, b) {
        const intersection = a.getIntersection(b);
        if (!intersection)
            return [a];
        if (a.isEqual(b))
            return [];
        if (a.isInside(b))
            return [];
        if (b.isInside(a)) {
            const differences = [];
            if (!a.startIsEqual(b))
                differences.push(new FrameRange(a.start, b.start));
            if (!a.endIsEqual(b))
                differences.push(new FrameRange(b.end, a.end));
            return differences;
        }
        if (a.start < b.start)
            return [new FrameRange(a.start, b.start)];
        if (a.end > b.end)
            return [new FrameRange(b.end, a.end)];
        throw new Error(`FrameRange.difference should not reach this code! A:${a} - B:${b}`);
    }
    // #endregion SET[]
    static distinctFrameRanges(ranges) {
        // Step 1: Collect all start and end points
        const points = [];
        for (const range of ranges) {
            points.push(range.start);
            points.push(range.end);
        }
        // Step 2: Sort the points and remove duplicates
        const sortedPoints = Array.from(new Set(points)).sort((a, b) => {
            if (a < b)
                return -1;
            if (a > b)
                return 1;
            return 0;
        });
        // Step 3: Create partitions based on sorted points
        const partitions = [];
        for (let i = 0; i < sortedPoints.length - 1; i++) {
            const newRange = new FrameRange(sortedPoints[i], sortedPoints[i + 1]);
            const intersectAny = ranges.some((range) => range.isIntersecting(newRange));
            if (intersectAny)
                partitions.push(newRange);
        }
        return partitions;
    }
    static distinctPartition(segments, frameRangePropertyName = "absoluteRange") {
        // Step 1: Collect all start and end points
        const points = [];
        for (const segment of segments) {
            points.push(segment[frameRangePropertyName].start);
            points.push(segment[frameRangePropertyName].end);
        }
        // Step 2: Sort the points and remove duplicates
        const sortedPoints = Array.from(new Set(points)).sort((a, b) => {
            if (a < b)
                return -1;
            if (a > b)
                return 1;
            return 0;
        });
        // Step 3: Create partitions based on sorted points
        const partitions = [];
        for (let i = 0; i < sortedPoints.length - 1; i++) {
            const range = new FrameRange(sortedPoints[i], sortedPoints[i + 1]);
            const intersectAny = segments.some((segment) => segment[frameRangePropertyName].isIntersecting(range));
            if (intersectAny)
                partitions.push(range);
        }
        return partitions;
    }
}
_a = immerable;
export function millisecondsToFps(milliseconds, fps) {
    return Math.round((milliseconds / 1000) * fps);
}
export function framesToMilliseconds(frames, fps) {
    if (fps <= 0)
        return frames;
    return Math.round((frames * 1000) / fps);
}
//# sourceMappingURL=frameRange.js.map