import { describe, it, expect } from "vitest";
import { HasUsers } from "./hasUsers.js";
describe("HasUsers", () => {
    it("should add a new group when groups is initially null", () => {
        // Arrange
        const hasUsers = new HasUsers();
        const groupId = "testGroup";
        // Act
        const result = hasUsers.addGroup(groupId);
        // Assert
        expect(result).toBe(true);
        expect(hasUsers.groups).toEqual(["grp:testGroup"]);
    });
    it("should add a new group when groups is not empty", () => {
        // Arrange
        const initialGroups = ["grp:existingGroup"];
        const hasUsers = new HasUsers({ groups: initialGroups });
        const newGroupId = "newTestGroup";
        // Act
        const result = hasUsers.addGroup(newGroupId);
        // Assert
        expect(result).toBe(true);
        expect(hasUsers.groups).toEqual([...initialGroups, "grp:newTestGroup"]);
    });
    it("should not add a group if it already exists", () => {
        // Arrange
        const existingGroupId = "existingGroup";
        const hasUsers = new HasUsers({ groups: [`grp:${existingGroupId}`] });
        // Act
        const result = hasUsers.addGroup(existingGroupId);
        // Assert
        expect(result).toBe(false);
        expect(hasUsers.groups).toEqual([`grp:${existingGroupId}`]);
    });
    it("should remove an existing group", () => {
        // Arrange
        const groupIdToRemove = "groupToRemove";
        const hasUsers = new HasUsers({
            groups: [`grp:${groupIdToRemove}`, "grp:otherGroup"],
        });
        // Act
        const result = hasUsers.removeGroup(groupIdToRemove);
        // Assert
        expect(result).toBe(true);
        expect(hasUsers.groups).toEqual(["grp:otherGroup"]);
    });
    it("should not remove a group if it does not exist", () => {
        // Arrange
        const nonExistingGroupId = "nonExistingGroup";
        const initialGroups = ["grp:existingGroup1", "grp:existingGroup2"];
        const hasUsers = new HasUsers({ groups: initialGroups });
        // Act
        const result = hasUsers.removeGroup(nonExistingGroupId);
        // Assert
        expect(result).toBe(true); // Assuming method always returns true, as per current implementation
        expect(hasUsers.groups).toEqual(initialGroups);
    });
    it("should correctly initialize with partial IHasUsers data", () => {
        // Arrange
        const partialData = {
            users: ["user1", "user2"],
            groups: ["grp:group1"],
        };
        // Act
        const hasUsers = new HasUsers(partialData);
        // Assert
        expect(hasUsers.users).toEqual(partialData.users);
        expect(hasUsers.groups).toEqual(partialData.groups);
        expect(hasUsers.groupClaims).toEqual({});
    });
    it("should initialize with null values for users and groups when no data is provided", () => {
        // Arrange & Act
        const hasUsers = new HasUsers();
        // Assert
        expect(hasUsers.users).toEqual([]);
        expect(hasUsers.groups).toEqual([]);
        expect(hasUsers.groupClaims).toEqual({});
    });
});
//# sourceMappingURL=hasUsers.spec.js.map