import { describe, expect, it } from "vitest";
import { LocalCloudFile } from "./localCloudFile.js";
const keyEn = { languageCode: "en", version: 1 };
const keyDe = { languageCode: "de", version: 1 };
const keyIt = { languageCode: "it", version: 1 };
const nestedData = {
    filename: "variant-en.txt",
    localPath: "/local/path",
    cloudPath: "/cloud/path",
    cloudDownloadPath: "/download/path",
    fileMetadata: [{ key: "size", value: "2MB" }],
    jsonData: '{"key": "value"}',
    variants: [
        {
            key: keyDe,
            value: {
                filename: "deutsch.txt",
                variants: [
                    {
                        key: keyIt,
                        value: { filename: "italian.txt", variants: [] },
                    },
                ],
            },
        },
    ],
};
describe("LocalCloudFile", () => {
    it("should correctly initialize from given data", () => {
        // Arrange
        const data = {
            filename: "testfile.txt",
            localPath: "/local/path",
            cloudPath: "/cloud/path",
            cloudDownloadPath: "/download/path",
            fileMetadata: [{ key: "size", value: "2MB" }],
            jsonData: '{"key": "value"}',
            variants: [
                {
                    key: { languageCode: "en", version: 1 },
                    value: { filename: "variant-en.txt" },
                },
            ],
        };
        // Act
        const localCloudFile = LocalCloudFile.fromILocalCloudFile(data);
        // Assert
        expect(localCloudFile.variants.length).toBe(1);
        expect(localCloudFile.filename).toBe(data.filename);
        expect(localCloudFile.localPath).toBe(data.localPath);
        expect(localCloudFile.cloudPath).toBe(data.cloudPath);
        expect(localCloudFile.cloudDownloadPath).toBe(data.cloudDownloadPath);
        expect(localCloudFile.fileMetadata).toEqual(data.fileMetadata);
        expect(localCloudFile.jsonData).toBe(data.jsonData);
    });
    it("should handle missing properties in data for constructor", () => {
        // Arrange
        const data = {
            filename: "testfile.txt",
            // Other properties are intentionally left out
        };
        // Act
        const localCloudFile = LocalCloudFile.fromILocalCloudFile(data);
        // Assert
        expect(localCloudFile.filename).toBe(data.filename);
        expect(localCloudFile.localPath).toBeUndefined();
        expect(localCloudFile.cloudPath).toBeUndefined();
        expect(localCloudFile.cloudDownloadPath).toBeUndefined();
        expect(localCloudFile.fileMetadata).toBeUndefined();
        expect(localCloudFile.jsonData).toBeUndefined();
        expect(localCloudFile.variants.length).toBe(0);
    });
    describe("Variants", () => {
        it("should add a nested variant", () => {
            // Arrange
            const localCloudFile = new LocalCloudFile();
            const nestedLCF = LocalCloudFile.fromILocalCloudFile(nestedData);
            // Act
            localCloudFile.variantMap.add(keyEn, nestedLCF);
            // Assert
            expect(localCloudFile.variantMap.exists(keyEn)).toBe(true);
            const main = localCloudFile.variantMap.get(keyEn);
            expect(main === null || main === void 0 ? void 0 : main.filename).toEqual(nestedData.filename);
            expect(main === null || main === void 0 ? void 0 : main.variants.length).toBe(1);
            const sub1 = main === null || main === void 0 ? void 0 : main.variantMap.get(keyDe);
            expect(sub1 === null || sub1 === void 0 ? void 0 : sub1.variants.length).toBe(1);
            const sub2 = sub1 === null || sub1 === void 0 ? void 0 : sub1.variantMap.get(keyIt);
            expect(sub2 === null || sub2 === void 0 ? void 0 : sub2.variants.length).toBe(0);
        });
        it("should remove a variant by key", () => {
            // Arrange
            const localCloudFile = new LocalCloudFile();
            const key = { languageCode: "en", version: 1 };
            localCloudFile.variantMap.add(key, LocalCloudFile.fromILocalCloudFile({ filename: "variant-en.txt" }));
            // Act
            localCloudFile.removeVariant(key);
            // Assert
            expect(localCloudFile.variantMap.exists(key)).toBe(false);
            expect(localCloudFile.variants.length).toBe(0);
        });
    });
    describe("toILocalCloudFile", () => {
        it("should return a ILocalCloudFile object", () => {
            // Arrange
            const localCloudFile = LocalCloudFile.fromILocalCloudFile(nestedData);
            // Act
            const result = localCloudFile.toILocalCloudFile();
            // Assert
            expect(result).toEqual(nestedData);
        });
    });
});
//# sourceMappingURL=localCloudFile.spec.js.map