import { describe, it, expect, beforeEach } from "vitest";
import { DateRange } from "./dateRange";
describe("DateRange class tests", () => {
    let range;
    beforeEach(() => {
        range = new DateRange(new Date("2023-01-01"), new Date("2023-01-10"));
    });
    it("should initialize start and end correctly", () => {
        expect(range.start).toEqual(new Date("2023-01-01"));
        expect(range.end).toEqual(new Date("2023-01-10"));
    });
    it("should validate range correctly", () => {
        expect(range.isValid()).toBe(true);
        range = new DateRange(new Date("2023-01-10"), new Date("2023-01-01"));
        expect(range.isValid()).toBe(false);
    });
    it("should check if range is empty", () => {
        expect(range.isEmpty()).toBe(false);
        range = new DateRange(new Date("2023-01-01"), new Date("2023-01-01"));
        expect(range.isEmpty()).toBe(true);
    });
    it("should check if ranges are equal", () => {
        const other = new DateRange(new Date("2023-01-01"), new Date("2023-01-10"));
        expect(range.isEqual(other)).toBe(true);
        const different = new DateRange(new Date("2023-01-02"), new Date("2023-01-11"));
        expect(range.isEqual(different)).toBe(false);
    });
    it("should check if range is before another range", () => {
        const other = new DateRange(new Date("2023-01-11"), new Date("2023-01-20"));
        expect(range.isBefore(other)).toBe(true);
        expect(other.isBefore(range)).toBe(false);
    });
    it("should check if range is behind another range", () => {
        const other = new DateRange(new Date("2023-01-11"), new Date("2023-01-20"));
        expect(range.isBehind(other)).toBe(false);
        expect(other.isBehind(range)).toBe(true);
    });
    it("should check if ranges are intersecting", () => {
        const other = new DateRange(new Date("2023-01-05"), new Date("2023-01-15"));
        expect(range.isIntersecting(other)).toBe(true);
        const nonIntersecting = new DateRange(new Date("2023-01-11"), new Date("2023-01-20"));
        expect(range.isIntersecting(nonIntersecting)).toBe(false);
    });
    it("should check if range is inside another range", () => {
        const other = new DateRange(new Date("2022-12-01"), new Date("2023-02-01"));
        expect(range.isInside(other)).toBe(true);
        expect(other.isInside(range)).toBe(false);
    });
    it("should check if start points are equal", () => {
        const other = new DateRange(new Date("2023-01-01"), new Date("2023-01-20"));
        expect(range.startIsEqual(other)).toBe(true);
        const different = new DateRange(new Date("2023-01-02"), new Date("2023-01-20"));
        expect(range.startIsEqual(different)).toBe(false);
    });
    it("should check if end points are equal", () => {
        const other = new DateRange(new Date("2022-12-01"), new Date("2023-01-10"));
        expect(range.endIsEqual(other)).toBe(true);
        const different = new DateRange(new Date("2022-12-01"), new Date("2023-01-11"));
        expect(range.endIsEqual(different)).toBe(false);
    });
    it("should check if range contains a point", () => {
        expect(range.containsPoint(new Date("2023-01-05"))).toBe(true);
        expect(range.containsPoint(new Date("2022-12-31"))).toBe(false);
    });
    it("should calculate delta start correctly", () => {
        const other = new DateRange(new Date("2023-01-02"), new Date("2023-01-10"));
        expect(range.deltaStart(other).getTime()).toBe(-86400000); // Assuming delta is in days
    });
    it("should calculate delta end correctly", () => {
        const other = new DateRange(new Date("2023-01-01"), new Date("2023-01-11"));
        expect(range.deltaEnd(other).getTime()).toBe(-86400000); // Assuming delta is in days
    });
    it("should get intersection of ranges", () => {
        const other = new DateRange(new Date("2023-01-05"), new Date("2023-01-15"));
        const intersection = range.getIntersection(other);
        expect(intersection === null || intersection === void 0 ? void 0 : intersection.start).toEqual(new Date("2023-01-05"));
        expect(intersection === null || intersection === void 0 ? void 0 : intersection.end).toEqual(new Date("2023-01-10"));
    });
    it("should union ranges correctly", () => {
        const other = new DateRange(new Date("2023-01-05"), new Date("2023-01-15"));
        const union = range.union(other);
        expect(union.length).toBe(1);
        expect(union[0].start).toEqual(new Date("2023-01-01"));
        expect(union[0].end).toEqual(new Date("2023-01-15"));
    });
    it("should calculate difference of ranges", () => {
        const other = new DateRange(new Date("2023-01-05"), new Date("2023-01-15"));
        const difference = range.difference(other);
        expect(difference.length).toBe(1);
        expect(difference[0].start).toEqual(new Date("2023-01-01"));
        expect(difference[0].end).toEqual(new Date("2023-01-05"));
    });
    it("should shift range correctly", () => {
        const oneDayTicks = 86400000;
        const oneDay = new Date(oneDayTicks * 2);
        console.log(oneDay);
        const shifted = range.shift(oneDay); // Assuming shift is in days
        expect(shifted.start).toEqual(new Date("2023-01-03"));
        expect(shifted.end).toEqual(new Date("2023-01-12"));
    });
    it("should scale range correctly", () => {
        const scaled = range.scale(2); // Assuming scale factor
        expect(scaled.start).toEqual(new Date("2023-01-01"));
        expect(scaled.end).toEqual(new Date("2023-01-19"));
    });
});
//# sourceMappingURL=dateRange.spec.js.map