export class Range {
    constructor(start, end) {
        this.start = start;
        this.end = end;
    }
    // Since fromDto can't be abstract static, we'll use a generic static method
    // static fromDto<T>(
    //   dto: IRangeDto<T>,
    //   createInstance: (start: T, end: T) => Range<T>,
    //   deserializeValue: (value: any) => T
    // ): Range<T> {
    //   const start = deserializeValue(dto.start);
    //   const end = deserializeValue(dto.end);
    //   return createInstance(start, end);
    // }
    // Validation and Cloning
    isValid() {
        return this.compare(this.start, this.end) < 0;
    }
    isEmpty() {
        return this.compare(this.start, this.end) === 0;
    }
    isEqual(other) {
        return (this.compare(this.start, other.start) === 0 &&
            this.compare(this.end, other.end) === 0);
    }
    clone() {
        return this.createInstance(this.start, this.end);
    }
    // Comparison-based Methods
    isBefore(other) {
        return this.compare(this.end, other.start) <= 0;
    }
    isBehind(other) {
        return this.compare(this.start, other.end) >= 0;
    }
    isIntersecting(other) {
        return (this.compare(this.start, other.end) < 0 &&
            this.compare(other.start, this.end) < 0);
    }
    isInside(other) {
        return (this.compare(other.start, this.start) <= 0 &&
            this.compare(this.end, other.end) <= 0);
    }
    startIsEqual(other) {
        return this.compare(this.start, other.start) === 0;
    }
    endIsEqual(other) {
        return this.compare(this.end, other.end) === 0;
    }
    containsPoint(point) {
        return (this.compare(this.start, point) <= 0 && this.compare(point, this.end) <= 0);
    }
    // Arithmetic-based Methods (Implementations provided if arithmetic functions are defined)
    duration() {
        return this.subtract(this.end, this.start);
    }
    deltaStart(other) {
        return this.subtract(this.start, other.start);
    }
    deltaEnd(other) {
        return this.subtract(this.end, other.end);
    }
    getIntersection(other) {
        if (!this.isIntersecting(other))
            return null;
        const newStart = this.compare(this.start, other.start) > 0 ? this.start : other.start;
        const newEnd = this.compare(this.end, other.end) < 0 ? this.end : other.end;
        return this.createInstance(newStart, newEnd);
    }
    union(other) {
        const ranges = Array.isArray(other) ? other : [other];
        let result = [this.clone()];
        for (const range of ranges) {
            const temp = [];
            for (const existing of result) {
                if (existing.isIntersecting(range) ||
                    this.isAdjacent(existing, range)) {
                    const newStart = this.compare(existing.start, range.start) < 0
                        ? existing.start
                        : range.start;
                    const newEnd = this.compare(existing.end, range.end) > 0
                        ? existing.end
                        : range.end;
                    temp.push(this.createInstance(newStart, newEnd));
                }
                else {
                    temp.push(existing.clone());
                    temp.push(range.clone());
                }
            }
            result = temp;
        }
        // Remove duplicates and sort the result
        result = this.removeDuplicates(result);
        result.sort((a, b) => this.compare(a.start, b.start));
        return result;
    }
    difference(other) {
        if (!this.isIntersecting(other))
            return [this.clone()];
        const result = [];
        if (this.compare(this.start, other.start) < 0) {
            result.push(this.createInstance(this.start, other.start));
        }
        if (this.compare(this.end, other.end) > 0) {
            result.push(this.createInstance(other.end, this.end));
        }
        return result;
    }
    shift(amount) {
        const newStart = this.add(this.start, amount);
        const newEnd = this.add(this.end, amount);
        return this.createInstance(newStart, newEnd);
    }
    scale(factor) {
        const duration = this.subtract(this.end, this.start);
        const newDuration = this.multiply(duration, factor);
        const newEnd = this.add(this.start, newDuration);
        return this.createInstance(this.start, newEnd);
    }
    // Helper Methods
    isAdjacent(a, b) {
        return (this.compare(a.end, b.start) === 0 || this.compare(b.end, a.start) === 0);
    }
    removeDuplicates(ranges) {
        const uniqueRanges = [];
        for (const range of ranges) {
            if (!uniqueRanges.some((r) => r.isEqual(range))) {
                uniqueRanges.push(range);
            }
        }
        return uniqueRanges;
    }
}
//# sourceMappingURL=range.js.map