/* eslint-disable camelcase */
/* eslint-disable no-constant-condition */
import { RE_TIMESTAMP, parseTimestamps } from "./parseTimestamps.js";
export function stripBom(string) {
    if (typeof string !== "string") {
        throw new TypeError(`Expected a string, got ${typeof string}`);
    }
    // Catches EFBBBF (UTF-8 BOM) because the buffer-to-string
    // conversion translates it to FEFF (UTF-16 BOM).
    if (string.charCodeAt(0) === 0xfeff) {
        return string.slice(1);
    }
    return string;
}
export class Parser {
    constructor({ push }) {
        this.push = push;
        this.state = {
            expect: "header",
            row: 0,
            hasContentStarted: false,
            isWebVTT: false,
            node: {},
            buffer: [],
        };
    }
    isIndex(line) {
        return /^\d+$/.test(line.trim());
    }
    isTimestamp(line) {
        return RE_TIMESTAMP.test(line);
    }
    isVttComment(line) {
        return /^NOTE/.test(line);
    }
    getError(expected, index, row) {
        return new Error(`expected ${expected} at row ${index + 1}, but received: "${row}"`);
    }
    parseLine(line) {
        const contents = this.state.row === 0 ? stripBom(line) : line;
        if (!this.state.hasContentStarted) {
            if (contents.trim()) {
                this.state.hasContentStarted = true;
            }
            else {
                return;
            }
        }
        const parse = {
            header: this.parseHeader,
            id: this.parseId,
            timestamp: this.parseTimestamp,
            text: this.parseText,
            vtt_comment: this.parseVttComment,
        }[this.state.expect];
        parse.call(this, contents);
        this.state.row++;
    }
    flush() {
        if (this.state.buffer.length > 0) {
            this.pushNode();
        }
    }
    parseHeader(line) {
        if (!this.state.isWebVTT) {
            this.state.isWebVTT = /^WEBVTT/.test(line);
            if (this.state.isWebVTT) {
                this.state.node.type = "header";
            }
            else {
                this.parseId(line);
                return;
            }
        }
        this.state.buffer.push(line);
        if (!line) {
            this.state.expect = "id";
        }
    }
    parseId(line) {
        this.state.expect = "timestamp";
        if (this.state.node.type === "header") {
            this.pushNode();
        }
        if (this.isIndex(line))
            return;
        if (this.state.isWebVTT && this.isVttComment(line)) {
            this.state.expect = "vtt_comment";
            return;
        }
        this.parseTimestamp(line);
    }
    parseVttComment(line) {
        this.state.expect = "vtt_comment";
        if (line.trim() === "") {
            this.state.expect = "id";
        }
    }
    parseTimestamp(line) {
        if (!this.isTimestamp(line)) {
            throw this.getError("timestamp", this.state.row, line);
        }
        this.state.node = {
            type: "cue",
            data: Object.assign(Object.assign({}, parseTimestamps(line)), { text: "" }),
        };
        this.state.expect = "text";
    }
    parseText(line) {
        if (this.state.buffer.length === 0) {
            this.state.buffer.push(line);
            return;
        }
        if (this.isTimestamp(line)) {
            const lastIndex = this.state.buffer.length - 1;
            if (this.isIndex(this.state.buffer[lastIndex])) {
                this.state.buffer.pop();
            }
            this.pushNode();
            this.parseTimestamp(line);
            return;
        }
        if (this.isVttComment(line)) {
            this.pushNode();
            this.parseVttComment(line);
            return;
        }
        this.state.buffer.push(line);
    }
    pushNode() {
        if (this.state.node.type === "cue") {
            while (true) {
                const lastItem = this.state.buffer[this.state.buffer.length - 1];
                if (["", "\n"].includes(lastItem)) {
                    this.state.buffer.pop();
                }
                else {
                    break;
                }
            }
            while (true) {
                const firstItem = this.state.buffer[0];
                if (["", "\n"].includes(firstItem)) {
                    this.state.buffer.shift();
                }
                else {
                    break;
                }
            }
            this.state.node.data.text = this.state.buffer.join("\n");
        }
        if (this.state.node.type === "header") {
            this.state.node.data = this.state.buffer.join("\n").trim();
        }
        this.push(this.state.node);
        this.state.node = {};
        this.state.buffer = [];
    }
}
//# sourceMappingURL=parser.js.map