var _a;
import _ from "lodash";
import "reflect-metadata";
import { immerable } from "immer";
export const LOCKED = "LOCKED";
export class VariantMap {
    get default() {
        return this._default;
    }
    get keys() {
        return this.collection.map((item) => item.key);
    }
    get values() {
        return this.collection.map((item) => item.value);
    }
    get hasValues() {
        return this.collection.length > 0;
    }
    get length() {
        return this.collection.length;
    }
    constructor(map) {
        this[_a] = true; // Option 1
        this.collection = [];
        this._default = undefined;
        this.onChange = () => { };
        if (map && Array.isArray(map)) {
            this.initMap(map);
        }
    }
    setDefault(defaultValue) {
        this._default = defaultValue;
    }
    initMap(map) {
        this.collection = map;
    }
    exists(key) {
        return this.collection.some((item) => _.isEqual(item.key, key));
    }
    find(expression) {
        const variant = this.collection.find((item) => expression(item.key, item));
        return variant;
    }
    get(key, createIfMissing) {
        var _b, _c;
        const variant = this.collection.find((item) => _.isEqual(item.key, key));
        if (variant) {
            return variant.value;
        }
        if (createIfMissing) {
            const variant = (_b = this.collection.at(0)) !== null && _b !== void 0 ? _b : {
                key: {},
                value: this._default,
            };
            this.add(key, _.cloneDeep((_c = variant === null || variant === void 0 ? void 0 : variant.value) !== null && _c !== void 0 ? _c : {}));
            const result = this.collection.find((item) => _.isEqual(item.key, key));
            return result.value;
        }
        return null;
    }
    // #region modify
    /**
     * Adds a new key-value pair to the collection.
     *
     * @param key - The key to add to the collection.
     * @param value - The value associated with the key.
     * @returns A boolean indicating whether the addition was successful.
     * @throws {Error} If the key already exists in the collection.
     */
    add(key, value) {
        if (this.exists(key))
            throw new Error(`Key ${key} already exists`);
        this.collection.push({ key, value });
        if (this.onChange)
            this.onChange(this.collection);
        return true;
    }
    /**
     * Updates the value associated with the specified key in the collection.
     * If the key is found and the associated value is not locked, the value is updated.
     * If the key is found and the associated value is locked, the update is not performed.
     * If the key is not found, the update is not performed.
     *
     * @param key - The key of the item to update.
     * @param value - The new value to associate with the specified key.
     * @returns `true` if the update was successful, `false` otherwise.
     */
    update(key, value) {
        const variant = this.collection.find((item) => _.isEqual(item.key, key));
        if (variant && (variant === null || variant === void 0 ? void 0 : variant.value)[LOCKED] === true)
            return false;
        if (variant) {
            variant.value = value;
            if (this.onChange)
                this.onChange(this.collection);
            return true;
        }
        return false;
    }
    set(key, value) {
        const variant = this.collection.find((item) => _.isEqual(item.key, key));
        if (variant && (variant === null || variant === void 0 ? void 0 : variant.value)[LOCKED] === true)
            return false;
        return this.upsert(key, value);
    }
    upsert(key, value) {
        const variant = this.collection.find((item) => _.isEqual(item.key, key));
        if (variant && (variant === null || variant === void 0 ? void 0 : variant.value)[LOCKED] === true)
            return false;
        if (variant) {
            variant.value = value;
            if (this.onChange)
                this.onChange(this.collection);
            return true;
        }
        this.add(key, value);
        return true;
    }
    remove(key) {
        const variant = this.collection.find((item) => _.isEqual(item.key, key));
        if (variant && (variant === null || variant === void 0 ? void 0 : variant.value)[LOCKED] === true)
            return false;
        const index = this.collection.findIndex((item) => _.isEqual(item.key, key));
        if (index >= 0) {
            this.collection.splice(index, 1);
            if (this.onChange)
                this.onChange(this.collection);
            return true;
        }
        return false;
    }
    cloneItem(key, newKey) {
        if (this.exists(newKey))
            throw new Error(`Key ${newKey} already exists`);
        const variant = this.collection.find((item) => _.isEqual(item.key, key));
        if (variant) {
            const copy = _.cloneDeep(variant.value);
            if (copy[LOCKED]) {
                delete copy[LOCKED];
            }
            this.add(newKey, copy);
            return true;
        }
        return false;
    }
    cloneFromDefault(newKey) {
        if (this.exists(newKey))
            throw new Error(`Key ${newKey} already exists`);
        if (this._default) {
            this.add(newKey, _.cloneDeep(this._default));
            return true;
        }
        const variant = this.collection.at(0);
        if (variant) {
            this.add(newKey, _.cloneDeep(variant.value));
            return true;
        }
        return false;
    }
    // #endregion modify
    toIVariantArray() {
        return [...this.collection];
    }
    static fromArray(variants) {
        const variantMap = new VariantMap();
        variants.forEach((variant) => {
            const propertiesMap = new Map();
            variant.value.forEach((prop) => {
                propertiesMap.set(prop.name, prop);
            });
            variantMap.add(variant.key, propertiesMap);
        });
        return variantMap;
    }
    /**
     * Creates a `VariantMap` instance from an array of `IVariant` objects.
     *
     * @template K - The type of the key in the `VariantMap`.
     * @template T - The type of the value in the `VariantMap`.
     * @template KDto - The type of the key in the DTO `IVariant`.
     * @template TDto - The type of the value in the DTO `IVariant`.
     *
     * @param {IVariant<KDto, TDto>[]} variants - An array of `IVariant` objects to be converted.
     * @param {(item: IVariant<KDto, TDto>) => IVariant<K, T>} [dtoToMap=(item) => item as unknown as IVariant<K, T>] - A function to convert a DTO `IVariant` to a `VariantMap` `IVariant`.
     * @param {(item: IVariant<K, T>) => IVariant<KDto, TDto>} [mapToDto=(item) => item as unknown as IVariant<KDto, TDto>] - A function to convert a `VariantMap` `IVariant` to a DTO `IVariant`.
     * @returns {VariantMap<K, T>} A new `VariantMap` instance containing the converted `IVariant` objects.
     */
    static fromIVariantArray(variants, onChange = (variants) => { }, dtoToMap = (item) => item, mapToDto = (item) => item) {
        const map = new VariantMap(variants.map((item) => dtoToMap(item)));
        map.onChange = (collection) => {
            onChange(collection.map((item) => mapToDto(item)));
        };
        return map;
    }
}
_a = immerable;
export function fromIVariantArrayToVariantMap(variants) {
    const variantMap = new VariantMap();
    variants.forEach((variant) => {
        const propertiesMap = new Map();
        variant.value.forEach((prop) => {
            propertiesMap.set(prop.name, prop);
        });
        variantMap.add(variant.key, propertiesMap);
    });
    return variantMap;
}
//# sourceMappingURL=variantMap.js.map