/* eslint-disable max-nested-callbacks */
import { describe, it, expect } from "vitest";
import { VariantMap } from "./variantMap.js";
const keyEn = { languageCode: "en", version: 1 };
const defaultVariants = [
    {
        key: keyEn,
        value: "Initial Value",
    },
];
describe("VariantMap", () => {
    describe("Initialize", () => {
        it("should initialize an empty VariantMap", () => {
            // Arrange
            const variantMap = new VariantMap();
            // Act
            const isEmpty = variantMap.length === 0;
            // Assert
            expect(isEmpty).toBe(true);
        });
        it("should initialize a VariantMap with predefined variants", () => {
            // Arrange
            const initialVariants = [
                { key: { languageCode: "en", version: 1 }, value: "English v1" },
                { key: { languageCode: "fr", version: 1 }, value: "French v1" },
            ];
            // Act
            const variantMap = new VariantMap(initialVariants);
            const isInitialized = variantMap.length === initialVariants.length;
            // Assert
            expect(isInitialized).toBe(true);
        });
        it("should initialize the map with the given variants", () => {
            // Arrange
            const variantMap = new VariantMap();
            const initialVariants = [
                { key: { languageCode: "en", version: 1 }, value: "English v1" },
                { key: { languageCode: "fr", version: 1 }, value: "French v1" },
            ];
            // Act
            variantMap.initMap(initialVariants);
            // Assert
            expect(variantMap.length).toBe(initialVariants.length);
            expect(variantMap.get({ languageCode: "en", version: 1 })).toBe("English v1");
            expect(variantMap.get({ languageCode: "fr", version: 1 })).toBe("French v1");
        });
    });
    describe("Read", () => {
        describe("exist", () => {
            it("should return true if a key exists", () => {
                // Arrange
                const variantMap = new VariantMap();
                const key = { languageCode: "en", version: 1 };
                variantMap.add(key, "English v1");
                // Act
                const keyExists = variantMap.exists(key);
                // Assert
                expect(keyExists).toBe(true);
            });
            it("should return false if a key does not exist", () => {
                // Arrange
                const variantMap = new VariantMap();
                const key = { languageCode: "en", version: 1 };
                // Act
                const keyExists = variantMap.exists(key);
                // Assert
                expect(keyExists).toBe(false);
            });
        });
        describe("find", () => {
            it("should find an item based on a custom expression", () => {
                // Arrange
                const variantMap = new VariantMap();
                variantMap.add({ languageCode: "en", version: 1 }, "English v1");
                variantMap.add({ languageCode: "fr", version: 1 }, "French v1");
                // Act
                const foundItem = variantMap.find((key, item) => key.languageCode === "fr" && item.value.includes("French v1"));
                // Assert
                expect(foundItem).toEqual({
                    key: { languageCode: "fr", version: 1 },
                    value: "French v1",
                });
            });
            it("should return undefined if no item matches the expression", () => {
                // Arrange
                const variantMap = new VariantMap();
                variantMap.add({ languageCode: "en", version: 1 }, "English v1");
                // Act
                const foundItem = variantMap.find((key, item) => key.languageCode === "de");
                // Assert
                expect(foundItem).toBeUndefined();
            });
        });
        describe("get", () => {
            it("should retrieve the value for an existing key", () => {
                // Arrange
                const variantMap = new VariantMap();
                const key = { languageCode: "en", version: 1 };
                const expectedValue = "English v1";
                variantMap.add(key, expectedValue);
                // Act
                const retrievedValue = variantMap.get(key);
                // Assert
                expect(retrievedValue).toBe(expectedValue);
            });
            it("should return null for a non-existing key", () => {
                // Arrange
                const variantMap = new VariantMap();
                const nonExistingKey = { languageCode: "de", version: 1 };
                // Act
                const result = variantMap.get(nonExistingKey);
                // Assert
                expect(result).toBeNull();
            });
            it("should create and return a default item if key is missing and createIfMissing is true", () => {
                // Arrange
                const variantMap = new VariantMap();
                const defaultValue = "Default Value";
                variantMap.setDefault(defaultValue);
                const newKey = { languageCode: "es", version: 1 };
                // Act
                const result = variantMap.get(newKey, true);
                // Assert
                expect(result).toBe(defaultValue);
                expect(variantMap.get(newKey)).toBe(defaultValue); // Check if the item was added
            });
        });
    });
    describe("Modify", () => {
        it("should set a default value", () => {
            // Arrange
            const variantMap = new VariantMap();
            const defaultValue = "Default Value";
            // Act
            variantMap.setDefault(defaultValue);
            // Assert
            // Assuming there's a way to access or check the default value.
            // If there isn't, this part of the test would need to be adjusted accordingly.
            expect(variantMap.default).toBe(defaultValue);
        });
        describe("add", () => {
            it("should add a new item to the collection", () => {
                // Arrange
                const variantMap = new VariantMap();
                const newKey = { languageCode: "en", version: 1 };
                const newValue = "English v1";
                let variants = variantMap.toIVariantArray();
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                // Act
                variantMap.add(newKey, newValue);
                // Assert
                expect(variantMap.get(newKey)).toBe(newValue);
                expect(variantMap.length).toBe(1);
                expect(variants.length).toBe(1);
            });
            it("should throw an error if the key already exists", () => {
                // Arrange
                const variantMap = new VariantMap();
                const existingKey = { languageCode: "en", version: 1 };
                variantMap.add(existingKey, "English v1");
                // Act & Assert
                expect(() => {
                    variantMap.add(existingKey, "English v2");
                }).toThrowError();
            });
        });
        describe("update", () => {
            it("should update the value for an existing key", () => {
                // Arrange
                const variantMap = new VariantMap();
                const key = { languageCode: "en", version: 1 };
                const originalValue = "English v1";
                const updatedValue = "English v2";
                variantMap.add(key, originalValue);
                let variants = variantMap.toIVariantArray();
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                // Act
                variantMap.update(key, updatedValue);
                // Assert
                expect(variantMap.get(key)).toBe(updatedValue);
                expect(variants[0].value).toBe(updatedValue);
            });
            it("should not change the collection if the key does not exist", () => {
                // Arrange
                const variantMap = new VariantMap();
                const nonExistingKey = { languageCode: "de", version: 1 };
                const value = "German v1";
                // Act
                variantMap.update(nonExistingKey, value);
                // Assert
                expect(variantMap.get(nonExistingKey)).toBeNull();
                expect(variantMap.length).toBe(0);
            });
        });
        describe("upsert", () => {
            it("should update an existing item", () => {
                // Arrange
                const variantMap = new VariantMap();
                const key = { languageCode: "en", version: 1 };
                const originalValue = "English v1";
                const updatedValue = "English v2";
                variantMap.add(key, originalValue);
                let variants = variantMap.toIVariantArray();
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                // Act
                variantMap.upsert(key, updatedValue);
                // Assert
                expect(variantMap.get(key)).toBe(updatedValue);
                expect(variants[0].value).toBe(updatedValue);
            });
            it("should insert a new item if the key does not exist", () => {
                // Arrange
                const variantMap = new VariantMap();
                const newKey = { languageCode: "es", version: 1 };
                const newValue = "Spanish v1";
                let variants = variantMap.toIVariantArray();
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                // Act
                variantMap.upsert(newKey, newValue);
                // Assert
                expect(variantMap.get(newKey)).toBe(newValue);
                expect(variantMap.length).toBe(1);
                expect(variants.length).toBe(1);
            });
            it("should update an existing item with upsert", () => {
                // Arrange
                let variants = [...defaultVariants];
                const variantMap = new VariantMap([
                    ...variants,
                ]);
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                const updatedValue = "Updated Value";
                // Act
                variantMap.upsert(keyEn, updatedValue);
                // Assert
                expect(variantMap.get(keyEn)).toBe(updatedValue);
                expect(variants[0].value).toBe(updatedValue);
            });
        });
        describe("remove", () => {
            it("should remove an item by key", () => {
                // Arrange
                const variantMap = new VariantMap();
                const key = { languageCode: "en", version: 1 };
                variantMap.add(key, "English v1");
                let variants = variantMap.toIVariantArray();
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                // Act
                variantMap.remove(key);
                // Assert
                expect(variantMap.get(key)).toBeNull();
                expect(variantMap.length).toBe(0);
                expect(variants.length).toBe(0);
            });
            it("should not alter the collection if the key does not exist", () => {
                // Arrange
                const variantMap = new VariantMap();
                const nonExistingKey = { languageCode: "de", version: 1 };
                // Act
                variantMap.remove(nonExistingKey);
                // Assert
                expect(variantMap.length).toBe(0);
            });
        });
        describe("clone", () => {
            it("should clone an item to a new key", () => {
                // Arrange
                const variantMap = new VariantMap();
                let variants = variantMap.toIVariantArray();
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                const originalKey = { languageCode: "en", version: 1 };
                const clonedKey = { languageCode: "en", version: 2 };
                const value = "English v1";
                variantMap.add(originalKey, value);
                // Act
                variantMap.cloneItem(originalKey, clonedKey);
                // Assert
                expect(variantMap.get(clonedKey)).toBe(value);
                expect(variantMap.length).toBe(2);
                expect(variants.length).toBe(2);
            });
            it("should throw an error if the new key already exists", () => {
                // Arrange
                const variantMap = new VariantMap();
                const originalKey = { languageCode: "en", version: 1 };
                const existingKey = { languageCode: "fr", version: 1 };
                variantMap.add(originalKey, "English v1");
                variantMap.add(existingKey, "French v1");
                // Act & Assert
                expect(() => {
                    variantMap.cloneItem(originalKey, existingKey);
                }).toThrowError(`Key ${existingKey} already exists`);
            });
            it("should clone from the default item to a new key", () => {
                // Arrange
                const variantMap = new VariantMap();
                let variants = [...defaultVariants];
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                const defaultValue = "Default Value";
                variantMap.setDefault(defaultValue);
                const newKey = { languageCode: "es", version: 1 };
                // Act
                variantMap.cloneFromDefault(newKey);
                // Assert
                expect(variantMap.get(newKey)).toBe(defaultValue);
                expect(variantMap.length).toBe(1);
                expect(variants.length).toBe(1);
            });
            it("should clone from the first item if default is not set", () => {
                // Arrange
                const variantMap = new VariantMap();
                let variants = [...defaultVariants];
                variantMap.onChange = (collection) => {
                    variants = collection.map((item) => {
                        return {
                            key: item.key,
                            value: item.value,
                        };
                    });
                };
                const firstItemKey = { languageCode: "en", version: 1 };
                const firstItemValue = "English v1";
                variantMap.add(firstItemKey, firstItemValue);
                const newKey = { languageCode: "es", version: 1 };
                // Act
                variantMap.cloneFromDefault(newKey);
                // Assert
                expect(variantMap.get(newKey)).toBe(firstItemValue);
                expect(variantMap.length).toBe(2);
                expect(variants.length).toBe(2);
            });
            describe("fromIVariantArray", () => {
                it("should create a VariantMap from an array of IVariant objects", () => {
                    // Arrange
                    const variants = [
                        { key: { id: 1 }, value: { name: "Item 1" } },
                        { key: { id: 2 }, value: { name: "Item 2" } },
                    ];
                    // Act
                    const variantMap = VariantMap.fromIVariantArray(variants);
                    // Assert
                    expect(variantMap.length).toBe(variants.length);
                    expect(variantMap.get({ id: 1 })).toEqual({ name: "Item 1" });
                    expect(variantMap.get({ id: 2 })).toEqual({ name: "Item 2" });
                });
                it("should use custom dtoToMap and mapToDto functions", () => {
                    // Arrange
                    let variants = [
                        { key: 1, value: "Item 1" },
                        { key: 2, value: "Item 2" },
                    ];
                    const dtoToMap = (item) => ({
                        key: item.key.toString(),
                        value: item.value,
                    });
                    const mapToDto = (item) => ({
                        key: parseInt(item.key, 10),
                        value: item.value,
                    });
                    // Act
                    const variantMap = VariantMap.fromIVariantArray(variants, (items) => {
                        variants = items;
                    }, dtoToMap, mapToDto);
                    // Assert
                    expect(variantMap.length).toBe(variants.length);
                    expect(variantMap.get("1")).toBe("Item 1");
                    expect(variantMap.get("2")).toBe("Item 2");
                });
                it("should update the original variants array when the map changes", () => {
                    // Arrange
                    // eslint-disable-next-line prefer-const
                    let variants = [
                        { key: 1, value: "Item 1" },
                        { key: 2, value: "Item 2" },
                    ];
                    const dtoToMap = (item) => ({
                        key: item.key.toString(),
                        value: item.value,
                    });
                    const mapToDto = (item) => ({
                        key: parseInt(item.key, 10),
                        value: item.value,
                    });
                    // Act
                    const variantMap = VariantMap.fromIVariantArray(variants, (items) => {
                        variants = items;
                    }, dtoToMap, mapToDto);
                    variantMap.add("3", "Item 3");
                    // Assert
                    expect(variants.length).toBe(3);
                    expect(variants[2]).toEqual({ key: 3, value: "Item 3" });
                });
            });
        });
    });
});
//# sourceMappingURL=variantMap.spec.js.map