var _a;
import "reflect-metadata";
import { immerable } from "immer";
import { PixelFormat, VideoCodecType, VideoFileType, VideoImageFormatType, VideoRatioType, VideoResolution, Aspect, } from "../enums/index.js";
import { DEFAULT_VIDEO_SETTINGS, } from "../interfaces/index.js";
export class VideoSettings {
    get iSize() {
        return {
            width: this.width,
            height: this.height,
        };
    }
    constructor(init) {
        this[_a] = true; // Option 1
        this.isLandscape = true;
        this.frameRate = 25;
        this.resolution = VideoResolution.HD;
        this.ratio = VideoRatioType.R16To9;
        this.ratioFraction = [16, 9];
        this.width = 1920;
        this.height = 1080;
        this.durationInFrames = 0;
        this.videoCodec = VideoCodecType.H264;
        this.videoImageFormat = VideoImageFormatType.JPEG;
        this.pixelFormat = PixelFormat.yuv420p;
        this.proresProfile = null;
        this.fileType = VideoFileType.mp4;
        this.alphaChannel = false;
        if (init)
            Object.assign(this, init);
    }
    aspectSize(size, aspect = Aspect.Fit) {
        const bigger = size.width > size.height ? size.width : size.height;
        const smaller = size.width < size.height ? size.width : size.height;
        switch (aspect) {
            case Aspect.Fit:
                return this.sizeWithRatio(smaller);
            case Aspect.Fill:
                return this.sizeWithRatio(bigger);
            default:
                return { width: size.width, height: size.height };
        }
    }
    sizeWithRatio(width) {
        switch (this.ratio) {
            case VideoRatioType.R9To16:
            case VideoRatioType.R16To9:
                if (this.isLandscape)
                    return { width, height: (width * 9) / 16 };
                return { width, height: (width * 16) / 9 };
            case VideoRatioType.R3To4:
            case VideoRatioType.R4To3:
                if (this.isLandscape)
                    return { width, height: (width * 3) / 4 };
                return { width, height: (width * 4) / 3 };
            case VideoRatioType.R1To1:
                return { width, height: width };
            default:
                return { width, height: width };
        }
    }
    calculateSize() {
        const size = VideoSettings.sizeFromVideoSettings(this);
        this.width = size.width;
        this.height = size.height;
    }
    toIVideoSettings() {
        return {
            isLandscape: this.isLandscape,
            frameRate: this.frameRate,
            resolution: this.resolution,
            ratio: this.ratio,
            ratioFraction: this.ratioFraction,
            width: this.width,
            height: this.height,
            durationInFrames: this.durationInFrames,
            videoCodec: this.videoCodec,
            videoImageFormat: this.videoImageFormat,
            pixelFormat: this.pixelFormat,
            proresProfile: this.proresProfile,
            fileType: this.fileType,
            alphaChannel: this.alphaChannel,
        };
    }
    static toSuffix(iSettings) {
        return VideoSettings.fromIVideoSettings(iSettings).toSuffix();
    }
    toSuffix() {
        return `${this.ratioToString()}-${this.resolution}`;
    }
    ratioToString() {
        return VideoSettings.ratioToString(this.ratioFraction, this.isLandscape);
    }
    // eslint-disable-next-line max-params
    static compatible(a, b, checkOrientation = true, checkRatio = true, checkResolution = true) {
        const orientation = VideoSettings.equalOrientation(a, b) || !checkOrientation;
        const ratio = VideoSettings.equalRatio(a, b) || !checkRatio;
        const resolution = VideoSettings.equalResolution(a, b) || !checkResolution;
        return {
            compatible: orientation && ratio && resolution,
            reason: `${orientation === false ? "Orientation" : ""} ${ratio === false ? "Ratio" : ""} ${resolution === false ? "Resolution" : ""}`,
        };
    }
    static equalOrientation(a, b) {
        return a.isLandscape === b.isLandscape;
    }
    static equalResolution(a, b) {
        return a.resolution === b.resolution;
    }
    static equalRatio(a, b) {
        return (a.ratioFraction[0] === b.ratioFraction[0] &&
            a.ratioFraction[1] === b.ratioFraction[1]);
    }
    static ratioToString(ratio, isLandscape) {
        const min = Math.min(ratio[0], ratio[1]);
        const max = Math.max(ratio[0], ratio[1]);
        return isLandscape ? `${max}to${min}` : `${min}to${max}`;
    }
    static fromLottieData(data) {
        const ratio = (data.w / data.h);
        const resolution = this.resolutionFromSize({
            width: data.w,
            height: data.h,
        });
        const settings = new VideoSettings({
            resolution,
            width: data.w,
            height: data.h,
            ratio,
            ratioFraction: VideoSettings.reduceFraction(data.w, data.h),
            frameRate: data.fr,
            durationInFrames: data.op - data.ip,
            isLandscape: ratio > 1,
        });
        return settings;
    }
    static fromIVideoSettings(data) {
        const settings = new VideoSettings(data);
        return settings;
    }
    // eslint-disable-next-line max-params
    static fromMetaData(width, height, ratio, frameRate, durationInSeconds) {
        const settings = new VideoSettings({
            width,
            height,
            ratio,
            ratioFraction: VideoSettings.reduceFraction(width, height),
            frameRate,
            durationInFrames: durationInSeconds * frameRate,
            isLandscape: ratio > 1,
        });
        return settings;
    }
    static default() {
        const settings = new VideoSettings(DEFAULT_VIDEO_SETTINGS);
        return settings;
    }
    static sizeFromVideoSettings(videoSettings) {
        const resolutions = VideoSettings.getResolutions(videoSettings.ratio);
        const resolution = resolutions.get(videoSettings.resolution);
        if (!resolution)
            return { width: 480, height: 360 };
        if (!videoSettings.isLandscape) {
            return { width: resolution.height, height: resolution.width };
        }
        return resolution;
    }
    static resolutionFromSize(size) {
        const ratio = (size.width / size.height);
        const resolutions = VideoSettings.getResolutions(ratio);
        for (const [key, value] of resolutions.entries()) {
            if (value.width === size.width && value.height === size.height) {
                return key;
            }
        }
        return VideoResolution.HD;
    }
    static getResolutions(ratio) {
        const map = new Map();
        switch (ratio) {
            case VideoRatioType.R16To9:
                map.set(VideoResolution.SD, { width: 1280, height: 720 });
                map.set(VideoResolution.QHD, { width: 1280, height: 720 });
                map.set(VideoResolution.HD, { width: 1920, height: 1080 });
                map.set(VideoResolution.UHD, { width: 3840, height: 2160 });
                map.set(VideoResolution._8K, { width: 7680, height: 4320 });
                break;
            case VideoRatioType.R1To1:
                map.set(VideoResolution.SD, { width: 720, height: 720 });
                map.set(VideoResolution.QHD, { width: 720, height: 720 });
                map.set(VideoResolution.HD, { width: 1080, height: 1080 });
                map.set(VideoResolution.UHD, { width: 2160, height: 2160 });
                map.set(VideoResolution._8K, { width: 4320, height: 4320 });
                break;
            case VideoRatioType.R4To3:
                map.set(VideoResolution.SD, { width: 640, height: 480 });
                map.set(VideoResolution.QHD, { width: 1920, height: 1440 });
                map.set(VideoResolution.HD, { width: 1920, height: 1440 });
                map.set(VideoResolution.UHD, { width: 3840, height: 2160 });
                map.set(VideoResolution._8K, { width: 7680, height: 5760 });
                break;
            default:
        }
        return map;
    }
    static getThumbnailResolutions(ratio, isLandscape) {
        switch (ratio) {
            case VideoRatioType.R16To9:
                return isLandscape
                    ? { width: 640, height: 360 }
                    : { width: 360, height: 640 };
            case VideoRatioType.R1To1:
                return { width: 640, height: 640 };
            case VideoRatioType.R4To3:
                return isLandscape
                    ? { width: 640, height: 480 }
                    : { width: 480, height: 640 };
            default:
                return isLandscape
                    ? { width: 640, height: 360 }
                    : { width: 360, height: 640 };
        }
    }
    static greatestCommonDivisor(a, b) {
        while (b) {
            const t = b;
            b = a % b;
            a = t;
        }
        return a;
    }
    // Function to find the smallest possible numerator, denominator pair
    static reduceFraction(numerator, denominator) {
        if (!numerator || !denominator)
            return [0, 1];
        numerator = Math.abs(numerator);
        denominator = Math.abs(denominator);
        const commonDivisor = VideoSettings.greatestCommonDivisor(numerator, denominator);
        return [numerator / commonDivisor, denominator / commonDivisor];
    }
    static getRatio(width, height) {
        if (!width || !height || height === 0)
            return 0;
        const ratio = width / height;
        return ratio;
    }
    static smallestDenominator(num) {
        // Extract the decimal part
        const decimalPart = num % 1;
        // Assuming maximum possible denominator as 1000000 for precision
        const denominator = 1000000;
        const numerator = Math.round(decimalPart * denominator);
        // Simplify the fraction using GCD
        const commonDivisor = this.greatestCommonDivisor(numerator, denominator);
        // Return the simplified denominator
        return denominator / commonDivisor;
    }
    static fractionFromNumber(num) {
        if (num <= 0)
            return [0, 1];
        const wholePart = Math.floor(num); // Extract the whole number part
        const decimalPart = num - wholePart; // Extract the decimal part
        // If the number is a whole number, return the number and 1 as the denominator
        if (decimalPart === 0)
            return [wholePart, 1];
        // Assuming maximum possible denominator as 1000000 for precision
        const denominator = 1000000;
        let numerator = Math.round(decimalPart * denominator);
        // Add the whole part to the numerator
        numerator += wholePart * denominator;
        // Simplify the fraction using GCD
        const commonDivisor = this.greatestCommonDivisor(numerator, denominator);
        // Return the simplified numerator and denominator as a tuple
        return [numerator / commonDivisor, denominator / commonDivisor];
    }
    static fractionFromVideoRatioType(ratio) {
        switch (ratio) {
            case VideoRatioType.R16To9:
                return [16, 9];
            case VideoRatioType.R9To16:
                return [9, 16];
            case VideoRatioType.R4To3:
                return [4, 3];
            case VideoRatioType.R3To4:
                return [3, 4];
            case VideoRatioType.R1To1:
                return [1, 1];
            default:
                return [0, 1];
        }
    }
    static differenece(a, b, properties) {
        const map = new Map();
        for (const key in a) {
            if (!properties || (properties === null || properties === void 0 ? void 0 : properties.includes(key))) {
                const propA = a[key];
                const propB = b[key];
                if (propA !== propB) {
                    map.set(key, { a: propA, b: propB });
                }
            }
        }
        return map;
    }
    static updateAfromB(a, b, properties) {
        const map = VideoSettings.differenece(a, b, properties);
        for (const [key, value] of map.entries()) {
            a[key] = value.b;
        }
        return a;
    }
}
_a = immerable;
//# sourceMappingURL=videoSettings.js.map