import { immerable } from "immer";
import type { IWebVTT, IWebVTTMinimal } from "../interfaces/index.js";
import { BoxModel } from "./boxModel.js";
import { FrameRange } from "./frameRange.js";
import { Cue, NodeCue, NodeList } from "./subtitle/index.js";
import { TimeRange } from "./timerange.js";
export interface ITranscriptionResult {
    duration: number;
    language: string;
    segments: ITranscriptionResultSegment[];
    text: string;
    task: string;
    subtitles?: ITranscriptionSubtitle[];
}
export interface ITranscriptionSubtitle {
    format: string;
    subtitles: string;
}
export interface ITranscriptionResultSegment {
    id: number;
    start: number;
    end: number;
    text: string;
    words?: ITranscriptionResultWord[];
    language?: string;
    confidence?: number;
    channel?: number;
    speaker?: number;
}
export interface ITranscriptionResultWord {
    start: number;
    end: number;
    word: string;
}
export interface CueChanges extends Cue {
    changedProperties: string[];
}
export declare class WebVTT {
    [immerable]: boolean;
    language: string;
    private nodeList;
    getNodeList(exclude?: "words"[]): NodeList;
    get cues(): NodeCue[];
    get endInSeconds(): number;
    constructor(obj?: Partial<WebVTT>);
    withFrameTiming(fps: number, offset?: number): WebVTT;
    inRange(range: FrameRange): WebVTT;
    intersectRange(range: FrameRange, clamping?: boolean): WebVTT;
    private getFrameCues;
    getCueAtFrame(frame: number): NodeCue | undefined;
    addCue(node: Cue): void;
    addCueRange(nodes: Cue[]): void;
    removeScriptTags(): void;
    removeWords(): void;
    extractTexts(): string[];
    toIWebVTT(exclude?: "words"[]): IWebVTT;
    toIWebVTTMinimal(): IWebVTTMinimal;
    toJson(minimal?: boolean): string;
    toSrt(): string;
    toVtt(): string;
    toString(): string;
    toTimerangeTexts(minTextLength?: number): {
        text: string;
        timerange: TimeRange;
    }[];
    transform(): void;
    clearText(): void;
    overwriteSpatialProperties(prior: IWebVTT): void;
    clone(): WebVTT;
    difference(other: WebVTT): CueChanges[];
    compareCues(a: Cue, b: Cue): string[];
    static fromITranscriptionResult(transcript: ITranscriptionResult): WebVTT;
    static fromString(webVTTString: string, language?: string): WebVTT | null;
    static fromHttp(source: string, language?: string, signal?: AbortSignal | undefined): Promise<WebVTT | null>;
    static fromIWebVTT(iWebVTT?: IWebVTT, language?: string): WebVTT;
    static fromJson(data: string): WebVTT;
    static fromCues(cues: Cue[], language?: string): WebVTT;
}
export type AutoKeyword = "auto";
export type LineAndPositionSetting = number | AutoKeyword;
export declare enum DirectionSetting {
    "" = "horizontal",
    "rl" = "rl",
    "lr" = "lr"
}
export declare enum LineAlignSetting {
    "start" = "start",
    "center" = "center",
    "end" = "end"
}
export declare enum PositionAlignSetting {
    "line-left" = "line-left",
    "center" = "center",
    "line-right" = "line-right",
    "auto" = "auto"
}
export declare enum AlignSetting {
    "start" = "start",
    "left" = "left",
    "center" = "center",
    "end" = "end",
    "right" = "right"
}
export declare class VTTCue {
    start: number;
    end: number;
    text: string;
    get htmlText(): string;
    region?: any;
    vertical?: DirectionSetting;
    snapToLines?: boolean;
    line?: LineAndPositionSetting;
    lineAlign?: LineAlignSetting;
    position?: LineAndPositionSetting;
    positionAlign?: PositionAlignSetting;
    size?: number;
    align?: AlignSetting;
    constructor(obj?: Partial<VTTCue>);
    static fromVtt(vtt: VTTCue): VTTCue;
    static splitSettings(cueSettings?: string): {
        key: string;
        value: string;
    }[];
    static splitSettingsToString(splits: {
        key: string;
        value: string;
    }[] | undefined): string;
    static parseCueSettings(start: number, end: number, text: string, cueSettings?: string | undefined): VTTCue;
    getBox(origin?: BoxModel): BoxModel;
    getTextAlign(): "start" | "end" | "left" | "right" | "center" | "justify" | "match-parent";
    set(key: string, v: any): void;
    get(key: string, dflt?: any, defaultKey?: any): any;
    has(k: string): boolean;
    alt(k: string, v: string, a: string[]): void;
    integer(k: string, v: string): void;
    percent(k: string, v: string): boolean;
}
export declare enum HorizontalAlignmentSettings {
    LEFT = "position:10%,line-left align:start",
    CENTER = "",
    RIGHT = "position:90%,line-right align:end"
}
export declare enum SizeSettings {
    SMALL = "size:45%",
    MEDIUM = "size:65%",
    LARGE = "size:80%"
}
export interface TimeRangeText {
    text: string;
    timerange: TimeRange;
}
export declare function mergeUntilMinLength(items: TimeRangeText[], minTextLength: number): TimeRangeText[];
//# sourceMappingURL=webVTT.d.ts.map